# -*- coding: utf-8 -*-
# .

__project_name__ = 'Defter'
__date__ = '18/May/2018'
__author__ = 'Erdinç Yılmaz'

from PySide6.QtCore import Signal, Qt, QSize, QItemSelectionModel
from PySide6.QtWidgets import QTreeView, QAbstractItemView, QHeaderView, QMenu, QProxyStyle
from PySide6.QtGui import QPen
from .treeSayfa import Sayfa


########################################################################
########################################################################
class YeniStil(QProxyStyle):

    def drawPrimitive(self, element, option, painter, widget=None):
        """ """
        if element == QProxyStyle.PrimitiveElement.PE_IndicatorItemViewItemDrop and not option.rect.isNull():
            painter.setPen(QPen(Qt.GlobalColor.blue, 3, Qt.PenStyle.DotLine))
            # option_new = QStyleOption(option)
            # option_new.rect.setLeft(0)
            # if widget:
            #     option_new.rect.setRight(widget.width())
            # option = option_new
        super().drawPrimitive(element, option, painter, widget)


#######################################################################
#######################################################################
class TreeView(QTreeView):
    f2Pressed = Signal()
    middleClick = Signal()
    # treewidgettaki, itemIsAboutToEdited
    # nesneAktiveEdildi = Signal(QTreeWidgetItem, int)  # itemActivated

    secimDegisti = Signal(Sayfa)

    boyut_degisti = Signal()

    # ---------------------------------------------------------------------
    def __init__(self, parent=None):

        super(TreeView, self).__init__(parent)
        # self.setAlternatingRowColors(True)
        self.setAnimated(True)
        # self.setIndentation(20)
        # self.setSortingEnabled(False)
        # self.setColumnHidden(1, False)
        # self.setColumnHidden(2, False)
        # self.setColumnHidden(3, False)
        self.setHeaderHidden(True)
        self.setDragEnabled(True)
        self.setAcceptDrops(True)
        self.setDropIndicatorShown(True)
        # self.setDragDropMode(QAbstractItemView.InternalMove)
        self.setDefaultDropAction(Qt.DropAction.MoveAction)
        self.setDragDropMode(QAbstractItemView.DragDropMode.DragDrop)
        self.setDragDropOverwriteMode(False)
        # self.setAutoExpandDelay()
        self.setItemsExpandable(True)

        # self.setRootIsDecorated(True)
        self.setUniformRowHeights(True)
        self.setIconSize(QSize(48, 48))
        # print(self.indentation())  # 20

        # self.header().setStretchLastSection(False)
        # self.header().setSectionResizeMode(QHeaderView.Interactive)
        self.header().setSectionResizeMode(QHeaderView.ResizeMode.ResizeToContents)
        # self.header().setSectionResizeMode(QHeaderView.Stretch)

        # doubleclick editi yoketmek icin
        # self.setEditTriggers(QAbstractItemView.EditKeyPressed)
        self.setEditTriggers(QAbstractItemView.EditTrigger.DoubleClicked)
        # self.setEditTriggers(QAbstractItemView.NoEditTriggers)
        self.setSelectionMode(QAbstractItemView.SelectionMode.SingleSelection)

        self.setSelectionBehavior(QAbstractItemView.SelectionBehavior.SelectRows)

        self.setVerticalScrollMode(QAbstractItemView.ScrollMode.ScrollPerPixel)
        self.verticalScrollBar().setSingleStep(7)

        self.setAutoScroll(True)
        # self.resizeColumnToContents(0)
        # self.setAutoScrollMargin(20)

        # self.setContextMenuPolicy()

        self.setStyle(YeniStil())
        # css_stil = 'QTreeView::item {margin-bottom: 20px;} QTreeView::item:!has-children {margin-bottom: 20px;}'
        # self.setStyleSheet(css_stil)

    # ---------------------------------------------------------------------
    def contextMenuEvent(self, event):
        menu = QMenu(self)
        menu.addSection("Boyut")
        actionListe = menu.addAction("Liste")
        actionResimKucuk = menu.addAction("Küçük")
        actionResimOrta = menu.addAction("Orta")
        actionResimBuyuk = menu.addAction("Buyuk")
        # menu.addSeparator()
        # actionKopyala = menu.addAction("Çoğalt")
        # actionOzellikler = menu.addAction("Ozellikler")
        # menu.addSection("Resmet")
        # actionGozuken = menu.addAction("Gozuken")
        # actionGozuken = menu.addAction("Tum Sahne")
        action = menu.exec_(self.mapToGlobal(event.pos()))

        if action == actionListe:
            size = QSize(20, 20)
            self.setIconSize(size)
            self.model().treeViewIkonBoyutu = size
        elif action == actionResimKucuk:
            size = QSize(48, 48)
            self.setIconSize(size)
            self.model().treeViewIkonBoyutu = size
        elif action == actionResimOrta:
            size = QSize(128, 128)
            self.setIconSize(size)
            self.model().treeViewIkonBoyutu = size
        elif action == actionResimBuyuk:
            size = QSize(256, 256)
            self.setIconSize(size)
            self.model().treeViewIkonBoyutu = size

    # ---------------------------------------------------------------------
    def mousePressEvent(self, e):
        self.draggedItemIndex = self.indexAt(e.pos())
        super(TreeView, self).mousePressEvent(e)

    # ---------------------------------------------------------------------
    def currentChanged(self, yeniIdx, eskiIdx):

        self.secimDegisti.emit(self.model().sayfa_indexten(yeniIdx))

    # ---------------------------------------------------------------------
    def selectionChanged(self, selected, deselected):
        """This slot is called when the selection is changed.
        The previous selection (which may be empty),
        is specified by deselected , and the new selection by selected ."""
        #  to disable ctrl deselect, and empty click deselect
        # print(selected.isEmpty(), deselected.isEmpty())
        if selected.isEmpty():
            # selected.clear()
            # deselected.clear()
            return

        # if not deselected.isEmpty():
        #     # selected.clear()
        #     # deselected.clear()
        #     return
        super(TreeView, self).selectionChanged(selected, deselected)
        # print(QItemSelection.indexes())
        # self.secimDegisti.emit(self.model().itemFromIndex(QItemSelection),
        #                        self.model().itemFromIndex(QItemSelection_1))
        # TODO: burda niye yukardaki selected kullanamadık?
        sayfa = self.get_selected_sayfa()
        if sayfa:
            self.secimDegisti.emit(sayfa)
            # TODO: performans optimize et, direkt inde mi kullanmak ,veya key pressed ile secilince scroll etmece
            # bi de gerek yok zaten
            self.scrollTo(self.model().index_sayfadan(sayfa),
                          QTreeView.ScrollHint.EnsureVisible)  # PositionAtCenter

    # # ---------------------------------------------------------------------
    # def mouseDoubleClickEvent(self, QMouseEvent):
    #     self.itemIsAboutToEdited.emit(self.currentItem().text(0))
    #     super(TreeView, self).mouseDoubleClickEvent(QMouseEvent)
    #
    # # ---------------------------------------------------------------------
    # def keyPressEvent(self, QKeyEvent):
    #     if QKeyEvent.key() == Qt.Key_F2:
    #         self.itemIsAboutToEdited.emit(self.currentItem().text(0))
    #     super(TreeView, self).keyPressEvent(QKeyEvent)

    # ---------------------------------------------------------------------
    def dropEvent(self, event):
        # if event.source():
        #     print(event.source())
        #     QTreeView.dropEvent(self, event)
        # else:

        hedefParentIndex = self.indexAt(event.pos())
        dropIndicatorPos = self.dropIndicatorPosition()
        # print(dropIndicatorPos)

        if not hedefParentIndex.isValid():
            hedefParentIndex = self.rootIndex()

        # if not hedefIndex.parent().isValid() and  hedefIndex.row() != -1:
        if dropIndicatorPos == self.DropIndicatorPosition.OnItem:
            hedefRow = -11
        elif dropIndicatorPos == self.DropIndicatorPosition.BelowItem:
            # bu ikisinde item hedef degil aslinda, (AboveItem, BelowItem)
            # hedefParentIndex demek anlam karmasasi yaratmasin, bu durumda parent bunun parenti
            # asagida var zaten.
            # kenarinda bulunmak suretiyle, uzerinde bulundugumuz item,
            # indexAt ile bu itemi bulu sirasini bulup alt veya ust cizgi olmasina gore
            # bir row index belirliyoruz. bu moverRowstada degisecek cunku item silinebilir
            # sira degisebilir. bu silinmeden onceki ham hal.
            hedefRow = hedefParentIndex.row()
            hedefParentIndex = hedefParentIndex.parent()
        elif dropIndicatorPos == self.DropIndicatorPosition.AboveItem:
            hedefRow = hedefParentIndex.row() - 1
            hedefParentIndex = hedefParentIndex.parent()
        elif dropIndicatorPos == self.DropIndicatorPosition.OnViewport:
            hedefRow = -22

        # m = event.mimeData()
        # if m.hasUrls():
        # print("dropped")
        # selectedIndex = self.selectedIndexes()[0]
        # print(self.draggedItemIndex)
        self.model().moveRows(self.draggedItemIndex.parent(), self.draggedItemIndex.row(), 1, hedefParentIndex,
                              hedefRow)
        # event.acceptProposedAction()
        # event.accept()
        event.ignore()

    # ---------------------------------------------------------------------
    def sayfa_bul_ve_sec(self, sayfa):
        # https://doc.qt.io/qt-5/qt.html#MatchFlag-enum
        # self.findItems(textToMatch, Qt.MatchContains | Qt.MatchRecursive)
        self._sayfa_bul_ve_sec(ustSayfa=self.model().kokSayfa, sayfa=sayfa)

    # ---------------------------------------------------------------------
    def _sayfa_bul_ve_sec(self, ustSayfa, sayfa):

        for s in ustSayfa.ic_sayfalar():
            if s == sayfa:
                self.sayfayi_sec_ve_current_index_yap(sayfa)
                break
            else:
                if s.ic_sayfalar():
                    self._sayfa_bul_ve_sec(s, sayfa)

    # ---------------------------------------------------------------------
    def sayfayi_sec_ve_current_index_yap(self, sayfa):

        # TODO: bu alttaki methodlarda cok biribiri arası zıplama var
        # optimize edemez miyiz acaba.. İnş. sonra tabi. ..

        # print("sayfa", sayfa)
        self.model().enSonAktifSayfa = sayfa
        idx = self.model().index_sayfadan(sayfa)
        # self.setCurrentIndex(idx) # bu ayni isi yapiyor alt iki satirin.
        self.selectionModel().select(idx, QItemSelectionModel.SelectionFlag.ClearAndSelect)
        self.selectionModel().setCurrentIndex(idx, QItemSelectionModel.SelectionFlag.ClearAndSelect)
        self.scrollTo(self.model().index_sayfadan(sayfa), QTreeView.ScrollHint.EnsureVisible)  # PositionAtCenter

        # print(self.model().enSonAktifSayfa.adi)

    # ---------------------------------------------------------------------
    def sayfayi_current_index_yap(self, sayfa):
        # self.setCurrentIndex(self.model().indexFromItem(sayfa))
        idx = self.model().index_sayfadan(sayfa)
        self.selectionModel().setCurrentIndex(idx, QItemSelectionModel.SelectionFlag.ClearAndSelect)

    # # ---------------------------------------------------------------------
    # def sayfayi_secme(self, sayfa):
    #     self.selectionModel().select(self.model().sayfada_index(sayfa),
    #                                  self.selectionModel().Deselect)

    # ---------------------------------------------------------------------
    def sayfayi_expand_et(self, sayfa, ackapa=True):
        # print("expand")
        # idx = self.model().index_sayfadan(sayfa.parent())
        idx = self.model().index_sayfadan(sayfa)
        self.setExpanded(idx, ackapa)
        self.expand(idx)
        # self.update()

    # ------------------------------------------------------------------------------
    def get_selected_sayfa(self):
        if self.selectedIndexes():
            sayfa = self.model().sayfa_indexten(self.selectedIndexes()[0])
            # print(sayfa, "burda")
            return sayfa

        # else:
        #     return self.get_current_sayfa()
        # return None
        return self.model().kokSayfa

    # ------------------------------------------------------------------------------
    def get_current_sayfa(self):
        sayfa = self.model().sayfa_indexten(self.currentIndex())
        return sayfa

    # ------------------------------------------------------------------------------
    def get_selected_sayfanin_parent_sayfasi(self):
        sayfa = self.get_selected_sayfa()
        if sayfa:
            if sayfa.parent():
                return sayfa.parent()

        return self.model().kokSayfa
        # return None

    # ------------------------------------------------------------------------------
    def mouseReleaseEvent(self, event):
        if event.button() == Qt.MouseButton.MiddleButton:
            self.middleClick.emit()
        return QTreeView.mouseReleaseEvent(self, event)

    # ------------------------------------------------------------------------------
    def keyPressEvent(self, event):

        if event.key() == Qt.Key.Key_F2:
            self.f2Pressed.emit()
            self.edit(self.currentIndex())
        else:
            return QTreeView.keyPressEvent(self, event)

    # ----------------------------------------------------------------------
    def zoom(self, delta):
        # print(delta)
        font = self.font()
        iconSize = self.iconSize()
        if delta < 0:
            size = font.pointSize() - 1
            if size > 4:
                font.setPointSize(size)
                self.setFont(font)
                self.itemDelegate().fontSize = size
                iconSize -= QSize(1, 1)
                self.setIconSize(iconSize)
                self.itemDelegate().iconSizeWidth = iconSize.width() + 9
        elif delta > 0:
            size = font.pointSize() + 1
            font.setPointSize(size)
            self.setFont(font)
            self.itemDelegate().fontSize = size
            iconSize += QSize(1, 1)
            self.setIconSize(iconSize)
            self.itemDelegate().iconSizeWidth = iconSize.width() + 9

    # ----------------------------------------------------------------------
    def wheelEvent(self, event):
        if event.modifiers() & Qt.KeyboardModifier.ControlModifier:
            self.zoom(event.angleDelta().y())  # pixelDelta
            # print(event.angleDelta().y())  # dikey delta = y
        else:
            QTreeView.wheelEvent(self, event)



# -*- coding: utf-8 -*-
# .

__project_name__ = 'Defter'
__author__ = 'Erdinç Yılmaz'
__date__ = '20/Aug/2016'

from urllib.error import HTTPError
# from urllib.request import urlretrieve
from urllib.request import URLopener
from PySide6.QtCore import QObject, Slot, Signal, QPointF


# from PyQt5.QtGui import QIcon

# ########################################################################
# class IkonOlusturucuWorker(QObject):
#     ikon_olustur_sinyal = Signal(object, int)
#     ikon_olusturuldu = Signal(QIcon)
#
#     # ---------------------------------------------------------------------
#     def __init__(self, parent=None):
#         super(IkonOlusturucuWorker, self).__init__(parent)
#
#         self.ikon_olustur_sinyal.connect(self.act_ikon_olustur)
#
#     # ---------------------------------------------------------------------
#     @Slot(object, int)
#     def act_ikon_olustur(self, sayfa, tree_view_genislik):
#         view = sayfa.view
#         if view:
#             pixmap = view.grab(view.viewport().rect())
#             # pixmap = pixmap.scaled(128,128,Qt.KeepAspectRatioByExpanding, Qt.FastTransformation)
#             pixmap = pixmap.scaledToWidth(tree_view_genislik, Qt.FastTransformation)
#
#             self.ikon_olusturuldu.emit(QIcon(pixmap))
#
#             # idx = treeView.currentIndex()
#             # treeView.dataChanged(idx, idx)
#             print("tamam")
#         else:
#             "olmadi"


########################################################################
class DownloadWorker(QObject):
    downloadWithThread = Signal(str, str, QPointF)

    finished = Signal(str, str, QPointF, QObject)
    failed = Signal(QObject)
    log = Signal(str, int, int)
    percentage = Signal(int)

    # ---------------------------------------------------------------------
    def __init__(self, parent=None):
        super(DownloadWorker, self).__init__(parent)

        self.downloadWithThread.connect(self.download)

    # ---------------------------------------------------------------------
    @Slot(str, str, QPointF)
    def download(self, url, imageSavePath, scenePos):

        try:
            self.log.emit("Downloading image from: {}".format(url), 5000, 0)
            # urlretrieve(url, imageSavePath, reporthook=self.report)
            opener = URLopener()
            opener.addheader('User-Agent', 'Mozilla/5.0 (Windows NT 6.1; WOW64; rv:40.0) Gecko/20100101 Firefox/40.1')
            # filename, headers = opener.retrieve(url, imageSavePath, reporthook=self.report)
            opener.retrieve(url, imageSavePath, reporthook=self.report)

            self.log.emit("Image succesfully downloaded", 5000, 1)
            self.finished.emit(url, imageSavePath, scenePos, self)

        except HTTPError as e:
            self.log.emit("Could not load image: {} ({})".format(url, e), 5000, 2)
            self.failed.emit(self)

    # ---------------------------------------------------------------------
    def report(self, count, blockSize, totalSize):
        percent = int(count * blockSize * 100 / totalSize)

        self.log.emit("\r{0:d}% complete".format(percent), 1000, 0)



# -*- coding: utf-8 -*-
# .

__project_name__ = 'Defter'
__date__ = '07/Sep/2017'
__author__ = 'Erdinç Yılmaz'

from contextlib import contextmanager


# ---------------------------------------------------------------------
@contextmanager
def signals_blocked_and_updates_disabled(widget):
    """
    """

    widget.setUpdatesEnabled(False)
    widget.blockSignals(True)
    yield
    widget.blockSignals(False)
    widget.setUpdatesEnabled(True)


# ---------------------------------------------------------------------
@contextmanager
def signals_blocked(widget):
    """
    """

    widget.blockSignals(True)
    yield
    widget.blockSignals(False)


# ---------------------------------------------------------------------
@contextmanager
def slot_disconnected(signal, slot):
    """
    """

    signal.disconnect(slot)
    yield
    signal.connect(slot)


# ---------------------------------------------------------------------
@contextmanager
def slot_disconnected_and_updates_disabled(signal, slot, widget):
    """
    """

    signal.disconnect(slot)
    widget.setUpdatesEnabled(False)
    yield
    signal.connect(slot)
    widget.setUpdatesEnabled(True)



# -*- coding: utf-8 -*-
# .

__project_name__ = 'Defter'
__author__ = 'Erdinç Yılmaz'
__date__ = '04/Mar/2017'

from collections import deque


########################################################################
class SelectionQueue(deque):
    # def __init__(self, scene, iterable=None, maxLen=None):
    #     super(SelectionQueue, self).__init__(iterable, maxLen)
    def __init__(self, scene):
        super(SelectionQueue, self).__init__()
        self.scene = scene

    # ---------------------------------------------------------------------
    def append(self, item):
        if self:
            self[-1].isActiveItem = False
            self[-1].update()
        item.isActiveItem = True
        self.scene.activeItem = item
        item.update()
        super(SelectionQueue, self).append(item)

    # ---------------------------------------------------------------------
    def remove(self, item):
        item.isActiveItem = False
        item.update()
        self.scene.activeItem = None
        super(SelectionQueue, self).remove(item)
        if self:
            self[-1].isActiveItem = True
            self[-1].update()
            self.scene.activeItem = self[-1]



# -*- coding: utf-8 -*-
# .

__project_name__ = 'Defter'
__author__ = 'Erdinç Yılmaz'
__date__ = '8/3/23'

from PySide6.QtCore import Qt, Signal
from PySide6.QtGui import QColor, QIcon
from PySide6.QtWidgets import QApplication, QPushButton
from . import icons_rc


#######################################################################
class RenkSecmeDugmesi(QPushButton):
    renkDegisti = Signal(QColor, str)
    # renk_secildi = Signal(QColor)
    esc_key_pressed = Signal()

    # ---------------------------------------------------------------------
    def __init__(self, parent=None):
        super(RenkSecmeDugmesi, self).__init__(parent)
        # self.setMouseTracking(True)
        self.setFocusPolicy(Qt.FocusPolicy.NoFocus)
        self.setFlat(True)
        self.setIcon(QIcon(":icons/renk-sec.png"))

        self.setFixedWidth(16)
        self.setFixedHeight(16)
        self.setToolTip(self.tr("Click and drag to select color from screen"))
        self.setStatusTip(self.tr("Click and drag to select color from screen"))

    # ---------------------------------------------------------------------
    def mousePressEvent(self, event):
        QApplication.setOverrideCursor(Qt.CursorShape.CrossCursor)
        super(RenkSecmeDugmesi, self).mousePressEvent(event)

    # ---------------------------------------------------------------------
    def mouseMoveEvent(self, event):

        pos = self.mapToGlobal(event.pos())
        ekran = QApplication.screenAt(pos)
        if not ekran:
            ekran = QApplication.primaryScreen()
        rect = ekran.geometry()
        pix = ekran.grabWindow(0, pos.x() - rect.x(), pos.y() - rect.y(), 1, 1)
        i = pix.toImage()
        col = i.pixelColor(0, 0)
        # bilgi = f"X: {pos.x()}  Y: {pos.y()}  -  R: {col.red()}  G: {col.green()}  B: {col.blue()}"
        bilgi = f"xy = ({pos.x()}, {pos.y()})    rgb=({col.red()}, {col.green()}, {col.blue()})"
        self.renkDegisti.emit(col, bilgi)

        super(RenkSecmeDugmesi, self).mouseMoveEvent(event)

    # ---------------------------------------------------------------------
    def mouseReleaseEvent(self, event):
        QApplication.setOverrideCursor(Qt.CursorShape.ArrowCursor)
        pos = self.mapToGlobal(event.pos())
        ekran = QApplication.screenAt(pos)
        if not ekran:
            ekran = QApplication.primaryScreen()
        rect = ekran.geometry()
        pix = ekran.grabWindow(0, pos.x() - rect.x(), pos.y() - rect.y(), 1, 1)
        i = pix.toImage()
        col = i.pixelColor(0, 0)
        bilgi = f"xy = ({pos.x()}, {pos.y()})    rgb=({col.red()}, {col.green()}, {col.blue()})"
        self.renkDegisti.emit(col, bilgi)

        super(RenkSecmeDugmesi, self).mouseReleaseEvent(event)

    # # ---------------------------------------------------------------------
    # def keyPressEvent(self, event):
    #
    #     if event.key() == Qt.Key.Key_Escape:
    #         # self.esc_key_pressed.emit()
    #         self.renkDegisti.emit(self.eski_renk)
    #     super(RenkSecmeDugmesi, self).keyPressEvent(event)
    #     self.close()
    #     self.deleteLater()



# -*- coding: utf-8 -*-
# .

__project_name__ = 'Defter'
__author__ = 'Erdinç Yılmaz'
__date__ = '20/12/21'

from PySide6.QtGui import (QBrush, QPainterPath, QPainterPathStroker)
from PySide6.QtWidgets import (QGraphicsItem)
from PySide6.QtCore import (QRectF, Qt)
from .. import shared


########################################################################
class YuvarlakFircaBoyutu(QGraphicsItem):
    Type = shared.YUVARLAK_FIRCA_BOYUTU_ITEM_TYPE

    # ---------------------------------------------------------------------
    def __init__(self, pos, rect, arkaPlanRengi, pen, parent=None):
        super(YuvarlakFircaBoyutu, self).__init__(parent)

        self._kim = shared.kim(kac_basamak=16)

        self.setPos(pos)
        self._rect = rect
        self._boundingRect = QRectF()

        self.secili_nesne_kalem_kalinligi = 0

        self.setFlags(QGraphicsItem.GraphicsItemFlag.ItemIsSelectable |
                      QGraphicsItem.GraphicsItemFlag.ItemIsMovable |
                      QGraphicsItem.GraphicsItemFlag.ItemIsFocusable)

        self._pen = pen
        self._brush = QBrush()

        self.cizgiRengi = pen.color()
        self.setCizgiRengi(self.cizgiRengi)  # also sets self._pen
        self.setArkaPlanRengi(arkaPlanRengi)  # also sets self._brush

    # ---------------------------------------------------------------------
    def type(self):
        # Enable the use of qgraphicsitem_cast with this item.
        return YuvarlakFircaBoyutu.Type

    # ---------------------------------------------------------------------
    def html_dive_cevir(self, html_klasor_kayit_adres, dosya_kopyalaniyor_mu):
        # Ola ki bu nesne aktif iken HTML olarak kaydet cagrilirsa diye
        # None dondurmeyelim stringe ekleniyor bu
        return ""

    # ---------------------------------------------------------------------
    def wheelEvent(self, event):
        # shift ile firca boyutu degistirirken, altta cizilen nesneyi dondurebiliyordu
        if event.modifiers() == Qt.KeyboardModifier.ShiftModifier:
            event.accept()
        # else:
        #     super(BaseItem, self).wheelEvent(event)

    # ---------------------------------------------------------------------
    def paint(self, painter, option, widget=None):

        if not self._pen.width():
            painter.setPen(Qt.PenStyle.NoPen)
        else:
            painter.setPen(self._pen)
        painter.setBrush(self._brush)
        painter.drawEllipse(self._rect)
        # painter.drawEllipse(self.boundingRect())

    # ---------------------------------------------------------------------
    def setRect(self, rect):
        if self._rect == rect:
            return
        self.prepareGeometryChange()
        self._rect = rect
        self._boundingRect = QRectF()
        self.update()

    # ---------------------------------------------------------------------
    def rect(self):
        return self._rect

    # ---------------------------------------------------------------------
    def setPen(self, pen):

        if self._pen == pen:
            return
        self.prepareGeometryChange()
        self._pen = pen
        self._boundingRect = QRectF()
        self.update(self.boundingRect())

    # ---------------------------------------------------------------------
    def pen(self):
        return self._pen

    # ---------------------------------------------------------------------
    def setBrush(self, brush):

        if self._brush == brush:
            return
        self._brush = brush
        self.update()

    # ---------------------------------------------------------------------
    def brush(self):
        return self._brush

    # ---------------------------------------------------------------------
    def setCizgiRengi(self, col):

        self._pen.setColor(col)
        self.cizgiRengi = col

        self.update()

    # ---------------------------------------------------------------------
    def setCizgiKalinligi(self, width):
        self._pen.setWidthF(width)
        self.update()

    # ---------------------------------------------------------------------
    def setArkaPlanRengi(self, col):
        self.arkaPlanRengi = col
        self.setBrush(QBrush(col))

    # ---------------------------------------------------------------------
    def shape(self):
        path = QPainterPath()
        if self._rect.isNull():
            return path
        # path = super(YuvarlakFircaBoyutu, self).shape()
        # path.addEllipse(self.boundingRect())

        path.addEllipse(self._rect)
        return self.qt_graphicsItem_shapeFromPath(path, self._pen)

    # ---------------------------------------------------------------------
    def boundingRect(self):
        self._boundingRect = QRectF(self.rect())
        return self._boundingRect

    # ---------------------------------------------------------------------
    def qt_graphicsItem_shapeFromPath(self, path, pen):

        # We unfortunately need this hack as QPainterPathStroker will set a width of 1.0
        # if we pass a value of 0.0 to QPainterPathStroker.setWidth()
        penWidthZero = 0.00000001
        if path == QPainterPath():
            return path
        ps = QPainterPathStroker()
        ps.setCapStyle(pen.capStyle())

        if pen.widthF() <= 0:
            ps.setWidth(penWidthZero)
        else:
            ps.setWidth(pen.widthF())
        ps.setJoinStyle(pen.joinStyle())
        ps.setMiterLimit(pen.miterLimit())
        p = ps.createStroke(path)  # return type: QPainterPath
        p.addPath(path)
        return p



# -*- coding: utf-8 -*-
# .

__project_name__ = 'Defter'
__date__ = '21/Oct/2017'
__author__ = 'Erdinç Yılmaz'

import os

from PySide6.QtCore import Qt, Slot, Signal, QSettings, QSize, QLocale
from PySide6.QtGui import QIcon, QKeySequence, QDoubleValidator, QIntValidator, QActionGroup, QAction, \
    QPageLayout
from PySide6.QtPrintSupport import QPrintPreviewWidget, QPrinter, QPrintDialog, QPageSetupDialog
from PySide6.QtWidgets import QVBoxLayout, QToolBar, QMainWindow, QLineEdit, QLabel, \
    QSizePolicy, QComboBox, QWidget, QFormLayout, QDockWidget, QPushButton, QHBoxLayout, QButtonGroup, \
    QRadioButton, QGroupBox, QCheckBox, QFileDialog, QApplication

from . import shared

from . import icons_rc


########################################################################
class ZoomValidator(QDoubleValidator):

    # ---------------------------------------------------------------------
    def __init__(self, bottom, top, decimals, parent=None):
        super(ZoomValidator, self).__init__(bottom, top, decimals, parent)

    # ---------------------------------------------------------------------
    def validate(self, text, pos):
        replacePercent = False
        if text.endswith('%'):
            text = text[:len(text) - 1]
            replacePercent = True

        state = QDoubleValidator.validate(self, text, pos)
        if replacePercent:
            text += '%'
        num_size = 4
        if state == QDoubleValidator.State.Intermediate:
            idx = text.find("{}".format(QLocale.system().decimalPoint()))
            if (idx == -1 and len(text) > num_size) or (idx != -1 and idx > num_size):
                return QDoubleValidator.State.Invalid

        # print(state)
        return state


# ########################################################################
# class LineEdit(QLineEdit):
#
#     # ---------------------------------------------------------------------
#     def __init__(self, parent=None):
#         super(LineEdit, self).__init__(parent)
#         self.setContextMenuPolicy(Qt.NoContextMenu)
#         self.returnPressed.connect(self.act_return_pressed)
#         self.originalText = ""
#
#     # ---------------------------------------------------------------------
#     def act_return_pressed(self):
#         self.originalText = self.text()
#
#     # ---------------------------------------------------------------------
#     def focusInEvent(self, QFocusEvent):
#         self.originalText = self.text()
#         QLineEdit.focusInEvent(self, QFocusEvent)
#
#     # ---------------------------------------------------------------------
#     def focusOutEvent(self, QFocusEvent):
#         if self.isModified() and not self.hasAcceptableInput():
#             self.setText(self.originalText)
#         QLineEdit.focusOutEvent(self, QFocusEvent)


########################################################################
class PrintPreviewDialog(QMainWindow):
    """
    custom print preivew dialog
    """

    paintRequested = Signal(QPrinter)
    previewChanged = Signal()
    paintRequestTypeChanged = Signal(str)

    # main window kullanip sonra setWindowModality

    # ---------------------------------------------------------------------
    def __init__(self, printer, yazdirmaAlaniTipi, parent=None):
        super(PrintPreviewDialog, self).__init__(parent)

        self.setAttribute(Qt.WidgetAttribute.WA_DeleteOnClose)
        # self.setWindowModality(Qt.WindowModal)
        # self.setWindowModality(Qt.ApplicationModal)

        # self.accepted.connect(self.yaz_ayarlar)

        self.printer = printer
        self.printDialog = None

        self.setGeometry(600, 150, 700, 800)

        self.olustur_ayarlar()
        self.oku_arayuz_ayarlari()

        self.create_preview_widget(printer)
        self.create_tool_bar()
        self.create_options_dock_widget(yazdirmaAlaniTipi)

        title = self.tr("Defter: Print Preview")
        if printer.docName():
            title = "{}: {}".format(title, printer.docName())
        self.setWindowTitle(title)

        if not printer.isValid():
            pass

            """#if defined(Q_OS_WIN) || defined(Q_OS_MAC)
                || printer->outputFormat() != QPrinter::NativeFormat
                #endif
            )
                pageSetupAction->setEnabled(false);"""
        self.preview.setFocus()

    # ---------------------------------------------------------------------
    def closeEvent(self, QCloseEvent):
        self.yaz_ayarlar()
        super(PrintPreviewDialog, self).closeEvent(QCloseEvent)

    # ---------------------------------------------------------------------
    def create_preview_widget(self, printer):
        centralWidget = QWidget(self)
        centralWidget.setContentsMargins(0, 0, 0, 0)
        layout = QVBoxLayout(centralWidget)  # no need to set layout if we give it a parent

        self.preview = QPrintPreviewWidget(printer, centralWidget)
        # preview.fitToWidth()
        # self.preview.fitInView()

        # self.preview.setFont(QFont('Sans', 9, QFont.Bold))

        # self.preview.setSinglePageViewMode()
        # self.preview.setViewMode(QPrintPreviewWidget.SinglePageView)

        # self.preview.paintRequested.connect(lambda: self.paint_requested(printer))
        self.preview.paintRequested.connect(lambda prntr: self.paintRequested.emit(prntr))
        self.preview.previewChanged.connect(self.act_preview_changed)

        self.setCentralWidget(centralWidget)
        layout.addWidget(self.preview)

    # ---------------------------------------------------------------------
    def create_options_dock_widget(self, yazdirmaAlaniTipi):
        self.optionsDW = QDockWidget(self)

        self.optionsDW.setWindowTitle(self.tr("Print Options"))
        self.optionsDW.setObjectName("printOptionsDockWidget")
        self.optionsDW.setAllowedAreas(Qt.DockWidgetArea.LeftDockWidgetArea | Qt.DockWidgetArea.RightDockWidgetArea)
        self.optionsDW.setFeatures(QDockWidget.DockWidgetFeature.DockWidgetMovable |
                                   QDockWidget.DockWidgetFeature.DockWidgetClosable |
                                   # QDockWidget.DockWidgetFeature.DockWidgetVerticalTitleBar|
                                   QDockWidget.DockWidgetFeature.DockWidgetFloatable
                                   )

        baseWidget = QWidget(self.optionsDW)
        baseLayout = QVBoxLayout(baseWidget)

        radioGBox = QGroupBox(baseWidget)
        radioGBox.setFlat(True)
        # radioGBox.setTitle("radio")
        radioGBoxLayout = QVBoxLayout(radioGBox)

        self.radioBtnGroup = QButtonGroup(radioGBox)
        self.radioDocument = QRadioButton(self.tr("Document"), radioGBox)
        self.radioPage = QRadioButton(self.tr("Page"), radioGBox)
        self.radioView = QRadioButton(self.tr("Current View"), radioGBox)
        self.radioSelection = QRadioButton(self.tr("Selection"), radioGBox)
        self.radioSelection.setEnabled(False)
        self.radioContent = QRadioButton(self.tr("Content"), radioGBox)
        self.radioContent.setEnabled(False)
        self.radioBtnGroup.addButton(self.radioDocument)
        self.radioBtnGroup.addButton(self.radioPage)
        self.radioBtnGroup.addButton(self.radioView)
        self.radioBtnGroup.addButton(self.radioSelection)
        self.radioBtnGroup.addButton(self.radioContent)

        radioGBoxLayout.addWidget(self.radioDocument)
        radioGBoxLayout.addWidget(self.radioPage)
        radioGBoxLayout.addWidget(self.radioView)
        radioGBoxLayout.addWidget(self.radioSelection)
        radioGBoxLayout.addWidget(self.radioContent)

        if yazdirmaAlaniTipi == "document":
            self.radioDocument.setChecked(True)
        elif yazdirmaAlaniTipi == "page":
            self.radioPage.setChecked(True)
        elif yazdirmaAlaniTipi == "view":
            self.radioView.setChecked(True)
        elif yazdirmaAlaniTipi == "selection":
            self.radioSelection.setChecked(True)
        elif yazdirmaAlaniTipi == "content":
            self.radioContent.setChecked(True)

        if self.parent().cScene.selectionQueue:
            self.radioSelection.setEnabled(True)
            if self.parent().cScene.activeItem.type() == shared.TEXT_ITEM_TYPE:
                self.radioContent.setEnabled(True)

        self.radioBtnGroup.buttonClicked.connect(self.act_radio_button_group_clicked)

        # ---------------------------------------------------------------------
        # options widgets

        self.cboxAsViewed = QCheckBox(self.tr("As viewed"), baseWidget)
        self.cboxAsViewed.toggled.connect(self.act_as_viewed_cbox_toggled)

        subOptionsGBox = QGroupBox(baseWidget)
        subOptionsGBox.setFlat(True)
        # optionsGBox.setTitle("options")
        subOptionsGBoxLayout = QVBoxLayout(subOptionsGBox)

        self.subOptionsRadioBtnGroup = QButtonGroup(subOptionsGBox)

        self.radioFitPage = QRadioButton(self.tr("Fit page"), subOptionsGBox)
        self.radioFitWidth = QRadioButton(self.tr("Fit width"), subOptionsGBox)
        self.radioFitPage.setChecked(True)

        self.subOptionsRadioBtnGroup.addButton(self.radioFitPage)
        self.subOptionsRadioBtnGroup.addButton(self.radioFitWidth)

        subOptionsGBoxLayout.addWidget(self.cboxAsViewed)
        subOptionsGBoxLayout.addWidget(self.radioFitPage)
        subOptionsGBoxLayout.addWidget(self.radioFitWidth)

        self.subOptionsRadioBtnGroup.buttonClicked.connect(self.act_sub_options_radio_button_toggled)

        # ---------------------------------------------------------------------
        # btns
        buttonLayout = QHBoxLayout()

        printBtn = QPushButton(self.tr("Print"), baseWidget)
        printBtn.clicked.connect(self.act_print)
        # if self.printer.outputFormat() != QPrinter.NativeFormat:
        #     printBtn.setEnabled(False)

        pdfBtn = QPushButton(self.tr("Export PDF"), baseWidget)
        pdfBtn.clicked.connect(self.act_export_pdf)

        # saveAsImageBtn = QPushButton(self.tr("Save as Image"), baseWidget)
        # saveAsImageBtn.clicked.connect(self.act_save_as_image)

        buttonLayout.addWidget(printBtn)
        buttonLayout.addWidget(pdfBtn)
        # buttonLayout.addWidget(saveAsImageBtn)

        closeBtn = QPushButton(self.tr("Close"), baseWidget)
        closeBtn.setShortcut(QKeySequence("Ctrl+Q"))
        closeBtn.clicked.connect(self.close)

        # ---------------------------------------------------------------------
        # genel
        baseLayout.addWidget(radioGBox)
        baseLayout.addWidget(subOptionsGBox)
        baseLayout.addSpacing(20)
        baseLayout.addLayout(buttonLayout)
        baseLayout.addStretch()
        baseLayout.addWidget(closeBtn)
        # baseLayout.addSpacing(10)

        self.optionsDW.setWidget(baseWidget)

        self.addDockWidget(Qt.DockWidgetArea.LeftDockWidgetArea, self.optionsDW)

    # ---------------------------------------------------------------------
    def act_as_viewed_cbox_toggled(self, secili_mi):
        # if secili_mi:
        #     pass

        # bu secili_mi sinyalini etrafta dolastirmaktansa, _paint methodlarında
        # self.cboxAsViewed secili durumu kontrol ediliyor,
        # burda tekrar paint methodunu cagiriyoruz
        self.act_radio_button_group_clicked(self.radioBtnGroup.checkedButton())

    # ---------------------------------------------------------------------
    def act_radio_button_group_clicked(self, btn):
        # TODO: buyuk dosyalar icin. end call edilsin hemen bitmemis ise.
        # QPrinter().paintingActive()
        # QPrinter().paintEngine().isActive()
        # painter active var bi de isactive de olabilir
        # print(self.printer.paintEngine().isActive())
        # print(self.printer.paintingActive())
        if self.printer.paintingActive():
            if self.printer.paintEngine().end():
                print("engine durduruldu")
        # TODO: buyuk dosyalar icin. end call edilsin hemen bitmemis ise.

        if btn == self.radioDocument:
            tip = "document"
        elif btn == self.radioPage:
            tip = "page"
        elif btn == self.radioView:
            tip = "view"
        elif btn == self.radioSelection:
            tip = "selection"
        elif btn == self.radioContent:
            tip = "content"

        self.paintRequestTypeChanged.emit(tip)
        self._update_options_group_box(tip)

    # ---------------------------------------------------------------------
    def act_sub_options_radio_button_toggled(self, btn):
        if btn == self.radioFitPage:
            tip = "fitPage"
        elif btn == self.radioFitWidth:
            tip = "fitWidth"

        # bu sinyali etrafta dolastirmaktansa, _paint methodlarında
        # hangi radyo buton secili kontrol ediyoruz
        # burda tekrar paint methodunu cagiriyoruz
        self.act_radio_button_group_clicked(self.radioBtnGroup.checkedButton())

    # ---------------------------------------------------------------------
    def _update_options_group_box(self, tip):

        if tip == "document":
            pass
        elif tip == "page":
            pass
        elif tip == "view":
            pass
        elif tip == "selection":
            pass
        elif tip == "content":
            pass

    # ---------------------------------------------------------------------
    def create_tool_bar(self):
        self.toolBar = QToolBar(self)
        self.toolBar.setObjectName("toolBar")
        self.toolBar.setWindowTitle("Tools")
        self.toolBar.setIconSize(QSize(16, 16))

        self.pageNumberLineEdit = QLineEdit(self.toolBar)
        self.pageNumberLineEdit.setAlignment(Qt.AlignmentFlag.AlignRight)
        self.pageNumberLineEdit.setSizePolicy(QSizePolicy.Policy.Fixed, QSizePolicy.Policy.Fixed)
        self.pageNumberLineEdit.editingFinished.connect(self.act_page_num_edited)

        self.pageNumberLabel = QLabel(self.toolBar)

        self.zoomCBox = QComboBox(self.toolBar)
        self.zoomCBox.setEditable(True)
        self.zoomCBox.setMinimumContentsLength(7)
        self.zoomCBox.setInsertPolicy(QComboBox.InsertPolicy.NoInsert)

        zoomEditorLineEdit = QLineEdit(self.toolBar)
        zoomEditorLineEdit.setValidator(ZoomValidator(1, 1000, 1, zoomEditorLineEdit))
        self.zoomCBox.setLineEdit(zoomEditorLineEdit)

        factors = (25, 50, 100, 200, 250, 300, 400, 800, 1600)
        for factor in factors:
            self.zoomCBox.addItem("{}%".format(factor))

        zoomEditorLineEdit.editingFinished.connect(self.act_zoom_factor_changed)
        self.zoomCBox.currentIndexChanged.connect(self.act_zoom_factor_changed)

        # fit group ---------------------------------------------------------------------
        # an action is added to a group by creating it with the group as its parent, no need to add it.
        self.fitGroup = QActionGroup(self.toolBar)
        self.actionFitWidth = QAction(QIcon(':icons/genislik-sigdir.png'), self.tr("Fit Width"), self.fitGroup)
        # self.actionFitWidthAction.setShortcut(QKeySequence("Q"))
        self.actionFitWidth.setCheckable(True)

        self.actionFitPage = QAction(QIcon(':icons/sayfa-sigdir.png'), self.tr("Fit Page"), self.fitGroup)
        # self.actionFitWidthAction.setShortcut(QKeySequence("Q"))
        self.actionFitPage.setCheckable(True)

        self.fitGroup.triggered.connect(self.act_fit)

        # zoom group ---------------------------------------------------------------------
        zoomGroup = QActionGroup(self.toolBar)
        self.actionZoomOut = QAction(QIcon(':icons/zoom-out.png'), self.tr("Zoom Out"), zoomGroup)
        # self.actionZoomOut.setShortcut(QKeySequence("Q"))
        self.actionZoomIn = QAction(QIcon(':icons/zoom-in.png'), self.tr("Zoom In"), zoomGroup)
        # self.actionZoomIn.setShortcut(QKeySequence("Q"))

        # orientation Group ---------------------------------------------------------------------
        orientationGroup = QActionGroup(self.toolBar)
        self.actionPortrait = QAction(QIcon(':icons/dikey-sayfa.png'), self.tr("Portrait"), orientationGroup)
        # self.actionPortrait.setShortcut(QKeySequence("Q"))
        self.actionPortrait.triggered.connect(self.preview.setPortraitOrientation)
        self.actionPortrait.setCheckable(True)

        self.actionLandscape = QAction(QIcon(':icons/yatay-sayfa.png'), self.tr("Landscape"), orientationGroup)
        # self.actionLandscape.setShortcut(QKeySequence("Q"))
        self.actionLandscape.triggered.connect(self.preview.setLandscapeOrientation)
        self.actionLandscape.setCheckable(True)

        # nav group ---------------------------------------------------------------------
        self.navGroup = QActionGroup(self.toolBar)
        self.navGroup.setExclusive(False)
        self.navGroup.triggered.connect(self.act_navigate)
        self.actionFirstPage = QAction(QIcon(':icons/first-page.png'), self.tr("First Page"), self.navGroup)
        # self.actionFirstPage.setShortcut(QKeySequence("Q"))
        self.actionPrevPage = QAction(QIcon(':icons/prev-page.png'), self.tr("Previous Page"), self.navGroup)
        # self.actionPrevPage.setShortcut(QKeySequence("Q"))
        self.actionNextPage = QAction(QIcon(':icons/next-page.png'), self.tr("Next Page"), self.navGroup)
        # self.actionNextPage.setShortcut(QKeySequence("Q"))
        self.actionLastPage = QAction(QIcon(':icons/last-page.png'), self.tr("Last Page"), self.navGroup)
        # self.actionLastPage.setShortcut(QKeySequence("Q"))

        # mode group ---------------------------------------------------------------------
        modeGroup = QActionGroup(self.toolBar)
        self.actionSingleMode = QAction(QIcon(':icons/single-page.png'), self.tr("Show single page"), modeGroup)
        # self.actionSingleMode.setShortcut(QKeySequence("Q"))
        self.actionSingleMode.setCheckable(True)

        self.actionFacingMode = QAction(QIcon(':icons/facing-pages.png'), self.tr("Show facing pages"), modeGroup)
        # self.actionFacingMode.setShortcut(QKeySequence("Q"))
        self.actionFacingMode.setCheckable(True)

        self.actionOverviewMode = QAction(QIcon(':icons/all-pages.png'),
                                          self.tr("Show overview of all pages"), modeGroup)
        # self.actionOverviewMode.setShortcut(QKeySequence("Q"))
        self.actionOverviewMode.setCheckable(True)
        modeGroup.addAction(self.actionOverviewMode)
        modeGroup.triggered.connect(self.act_set_mode)

        # printer group ---------------------------------------------------------------------
        printerGroup = QActionGroup(self.toolBar)
        self.actionPageSetup = QAction(QIcon(':icons/settings.png'), self.tr("Page Setup"), printerGroup)
        # self.actionPageSetup.setShortcut(QKeySequence("Q"))
        self.actionPageSetup.triggered.connect(self.act_page_setup)

        self.actionPrint = QAction(QIcon(':icons/print.png'), self.tr("Print"), printerGroup)
        # self.actionPrint.setShortcut(QKeySequence("Q"))
        self.actionPrint.triggered.connect(self.act_print)

        # page number widget---------------------------------------------------------------------
        pageEdit = QWidget(self.toolBar)
        vBoxLayout = QVBoxLayout()
        vBoxLayout.setContentsMargins(0, 0, 0, 0)

        # if mac

        pageNumberEditSize = self.pageNumberLineEdit.minimumSizeHint()
        pageNumberLabelSize = self.pageNumberLabel.minimumSizeHint()
        self.pageNumberLineEdit.resize(pageNumberEditSize)
        self.pageNumberLabel.resize(pageNumberLabelSize)
        # end if mac

        formLayout = QFormLayout()
        # if mac
        formLayout.setFieldGrowthPolicy(QFormLayout.FieldGrowthPolicy.AllNonFixedFieldsGrow)
        # end if
        formLayout.setWidget(0, QFormLayout.ItemRole.LabelRole, self.pageNumberLineEdit)
        formLayout.setWidget(0, QFormLayout.ItemRole.FieldRole, self.pageNumberLabel)
        vBoxLayout.addLayout(formLayout)
        vBoxLayout.setAlignment(Qt.AlignmentFlag.AlignCenter)
        pageEdit.setLayout(vBoxLayout)

        # ---------------------------------------------------------------------
        self.toolBar.addAction(self.actionFitWidth)
        self.toolBar.addAction(self.actionFitPage)
        self.toolBar.addSeparator()
        self.toolBar.addWidget(self.zoomCBox)
        self.toolBar.addAction(self.actionZoomOut)
        self.toolBar.addAction(self.actionZoomIn)
        self.toolBar.addSeparator()
        self.toolBar.addAction(self.actionPortrait)
        self.toolBar.addAction(self.actionLandscape)
        self.toolBar.addSeparator()
        self.toolBar.addAction(self.actionFirstPage)
        self.toolBar.addAction(self.actionPrevPage)
        self.toolBar.addWidget(pageEdit)
        self.toolBar.addAction(self.actionNextPage)
        self.toolBar.addAction(self.actionLastPage)
        self.toolBar.addSeparator()
        self.toolBar.addAction(self.actionSingleMode)
        self.toolBar.addAction(self.actionFacingMode)
        self.toolBar.addAction(self.actionOverviewMode)
        self.toolBar.addSeparator()
        self.toolBar.addAction(self.actionPageSetup)
        self.toolBar.addAction(self.actionPrint)

        self.zoomInBtn = self.toolBar.widgetForAction(self.actionZoomIn)
        self.zoomInBtn.setAutoRepeat(True)
        self.zoomInBtn.setAutoRepeatInterval(200)
        self.zoomInBtn.setAutoRepeatDelay(200)
        self.zoomInBtn.clicked.connect(self.act_zoom_in)

        self.zoomOutBtn = self.toolBar.widgetForAction(self.actionZoomOut)
        self.zoomOutBtn.setAutoRepeat(True)
        self.zoomOutBtn.setAutoRepeatInterval(200)
        self.zoomOutBtn.setAutoRepeatDelay(200)
        self.zoomOutBtn.clicked.connect(self.act_zoom_out)

        # -- baslangic --
        self.addToolBar(self.toolBar)
        self.actionFitPage.setChecked(True)
        self.actionSingleMode.setChecked(True)
        if self.preview.orientation() == QPageLayout.Orientation.Portrait:
            self.actionPortrait.setChecked(True)
        else:
            self.actionLandscape.setChecked(True)

    # ---------------------------------------------------------------------
    def _is_fitting(self):
        return self.fitGroup.isExclusive() and (self.actionFitWidth.isChecked() or self.actionFitPage.isChecked())

    # ---------------------------------------------------------------------
    def set_fitting(self, on):
        if self._is_fitting():
            return

        self.fitGroup.setExclusive(on)

        if on:
            if self.actionFitWidth.isChecked():
                action = self.actionFitWidth
            else:
                action = self.actionFitPage
            action.setChecked(True)
            if self.fitGroup.checkedAction() != action:
                self.fitGroup.removeAction(action)
                self.fitGroup.addAction(action)
        else:
            self.actionFitWidth.setChecked(False)
            self.actionFitPage.setChecked(False)

    # ---------------------------------------------------------------------
    def olustur_ayarlar(self):
        QApplication.setOrganizationName(shared.DEFTER_ORG_NAME)
        QApplication.setOrganizationDomain(shared.DEFTER_ORG_DOMAIN)
        QApplication.setApplicationName(shared.DEFTER_APP_NAME)
        self.settings = QSettings(shared.DEFTER_AYARLAR_DOSYA_ADRES, QSettings.Format.IniFormat)
        # self.settings.clear()

    # ---------------------------------------------------------------------
    def oku_arayuz_ayarlari(self):
        self.settings.beginGroup("PrintDialogSettings")
        if self.settings.contains("printWinGeometry"):
            self.restoreGeometry(self.settings.value("printWinGeometry"))
            self.restoreState(self.settings.value("printWinState", 0))
            # self.actionAlwaysOnTopToggle.blockSignals(True)
            # self.actionAlwaysOnTopToggle.setChecked(int(self.settings.value("alwaysOnTop", False)))
            # self.actionAlwaysOnTopToggle.blockSignals(False)
        self.settings.endGroup()

    # ---------------------------------------------------------------------
    def yaz_ayarlar(self):
        self.settings.beginGroup("PrintDialogSettings")
        self.settings.setValue("printWinGeometry", self.saveGeometry())
        self.settings.setValue("printWinState", self.saveState(0))  # version=0
        # self.settings.setValue("alwaysOnTop", _unicode_to_bool(self.actionAlwaysOnTopToggle.isChecked()))
        self.settings.endGroup()
        # self.settings.sync()

    # ---------------------------------------------------------------------
    @Slot(QPrinter)
    def paint_requested(self, painter):
        self.paintRequested.emit(painter)

    # ---------------------------------------------------------------------
    @Slot()
    def act_preview_changed(self):
        self.update_nav_actions()
        self.update_page_number_label()
        self.update_zoom_factor()

    # ---------------------------------------------------------------------
    def act_zoom_factor_changed(self):
        text = self.zoomCBox.lineEdit().text()
        # factor = float(text.translate(None, "%"))
        factor = float(text.replace("%", ""))
        factor = max(1.0, min(1000.0, factor))

        self.preview.setZoomFactor(factor / 100)
        self.zoomCBox.setEditText("{}%".format(factor))
        self.set_fitting(False)

    # ---------------------------------------------------------------------
    def update_nav_actions(self):
        cPage = self.preview.currentPage()
        pCount = self.preview.pageCount()

        self.actionNextPage.setEnabled(cPage < pCount)
        self.actionPrevPage.setEnabled(cPage > 1)
        self.actionFirstPage.setEnabled(cPage > 1)
        self.actionLastPage.setEnabled(cPage < pCount)
        self.pageNumberLineEdit.setText(str(cPage))

    # ---------------------------------------------------------------------
    def update_page_number_label(self):
        pCount = self.preview.pageCount()
        maxChars = len(str(pCount))
        self.pageNumberLabel.setText(" {}".format(pCount))
        # cyphersWidth = self.fontMetrics().width("8" * maxChars)
        cyphersWidth = self.fontMetrics().horizontalAdvance("8" * maxChars)
        maxWidth = self.pageNumberLineEdit.minimumSizeHint().width() + cyphersWidth
        self.pageNumberLineEdit.setMinimumWidth(maxWidth)
        self.pageNumberLineEdit.setMaximumWidth(maxWidth)
        self.pageNumberLineEdit.setValidator(QIntValidator(1, pCount, self.pageNumberLineEdit))

    # ---------------------------------------------------------------------
    def update_zoom_factor(self):
        self.zoomCBox.lineEdit().setText("{0:.2f}%".format(self.preview.zoomFactor() * 100))

    # ---------------------------------------------------------------------
    def act_fit(self, action):
        self.set_fitting(True)
        if action == self.actionFitPage:
            self.preview.fitInView()
        else:
            self.preview.fitToWidth()

    # ---------------------------------------------------------------------
    def act_zoom_in(self):
        self.set_fitting(True)
        self.preview.zoomIn()
        self.update_zoom_factor()

    # ---------------------------------------------------------------------
    def act_zoom_out(self):
        self.set_fitting(False)
        self.preview.zoomOut()
        self.update_zoom_factor()

    # ---------------------------------------------------------------------
    def act_page_num_edited(self):
        res = int(self.pageNumberLineEdit.text())
        self.preview.setCurrentPage(res)

    # ---------------------------------------------------------------------
    @Slot(QAction)
    def act_navigate(self, action):

        cPage = self.preview.currentPage()
        if action == self.actionPrevPage:
            self.preview.setCurrentPage(cPage - 1)
        elif action == self.actionNextPage:
            self.preview.setCurrentPage(cPage + 1)
        elif action == self.actionFirstPage:
            self.preview.setCurrentPage(1)
        elif action == self.actionLastPage:
            self.preview.setCurrentPage(self.preview.pageCount())
        self.update_nav_actions()

    # ---------------------------------------------------------------------
    def act_set_mode(self, action):
        if action == self.actionOverviewMode:
            self.preview.setViewMode(QPrintPreviewWidget.ViewMode.AllPagesView)
            self.set_fitting(False)
            self.fitGroup.setEnabled(False)
            self.navGroup.setEnabled(False)
            self.pageNumberLineEdit.setEnabled(False)
            self.pageNumberLabel.setEnabled(False)

        elif action == self.actionFacingMode:
            self.preview.setViewMode(QPrintPreviewWidget.ViewMode.FacingPagesView)

        elif action == self.actionSingleMode:
            self.preview.setViewMode(QPrintPreviewWidget.ViewMode.SinglePageView)

        if action == self.actionFacingMode or action == self.actionSingleMode:
            self.fitGroup.setEnabled(True)
            self.navGroup.setEnabled(True)
            self.pageNumberLineEdit.setEnabled(True)
            self.pageNumberLabel.setEnabled(True)
            self.set_fitting(True)

    # ---------------------------------------------------------------------
    def act_export_pdf(self):
        # if not self.printer.outputFormat()  == QPrinter.NativeFormat:

        fn = QFileDialog.getSaveFileName(self,
                                         self.tr('Export to PDF'),
                                         self.parent().sonKlasorDisaAktar,
                                         self.tr("*.pdf files (*.pdf);;All Files (*)"))

        path = fn[0]
        if path:
            if not path.endswith(".pdf"):
                path = '%s.pdf' % path
            self.printer.setOutputFileName(path)

            if self.printer.outputFileName():
                self.preview.print_()

            self.parent().log(self.tr("File succesfully saved! {}".format(path)), 7000)

            self.parent().sonKlasorDisaAktar = os.path.dirname(path)

            self.close()
            return

    # ---------------------------------------------------------------------
    def act_print(self):

        # if defined(Q_OS_WIN) || defined(Q_OS_MAC)
        # if not self.printer.outputFormat()  == QPrinter.NativeFormat:
        #     title = "Export to PDF"
        #     fn = QFileDialog.getSaveFileName(title,
        #                                            self.printer.outputFileName(),
        #                                            "*.pdf files (*.pdf);;All Files (*)")
        #
        #     path = fn[0]
        #     if path:
        #         if not path.endswith(".pdf"):
        #             path = '%s.pdf' % path
        #         self.printer.setOutputFileName(path)
        #
        #         if self.printer.outputFileName():
        #             self.preview.print()
        #         self.close()
        #         return

        # end if

        if not self.printDialog:
            self.printDialog = QPrintDialog(self.printer, self)
            # self.printDialog.set
        if self.printDialog.exec() == QPrintDialog.DialogCode.Accepted:
            self.preview.print_()
            self.close()

    # ---------------------------------------------------------------------
    def act_page_setup(self):
        pageSetupDialog = QPageSetupDialog(self.printer, self)
        if pageSetupDialog.exec() == QPageSetupDialog.DialogCode.Accepted:
            if self.preview.orientation() == QPageLayout.Orientation.Portrait:
                self.actionPortrait.setChecked(True)
                self.preview.setPortraitOrientation()
            else:
                self.actionLandscape.setChecked(True)
                self.preview.setLandscapeOrientation()

    # ---------------------------------------------------------------------
    def keyPressEvent(self, event):
        if event.key() == Qt.Key.Key_Escape:
            self.close()



# -*- coding: utf-8 -*-
# .

__project_name__ = 'Defter'
__author__ = 'Erdinç Yılmaz'
__date__ = '3/28/16'

import os
from PySide6.QtCore import Qt, QRectF, QPointF, QSizeF
from PySide6.QtGui import QColor, QPixmap, QTransform, QPixmapCache, QPainter
from PySide6.QtWidgets import QStyle
from ..nesneler.base import BaseItem
from ..nesneler.text import Text
from .. import shared


########################################################################
class Image(BaseItem):
    Type = shared.IMAGE_ITEM_TYPE

    # ---------------------------------------------------------------------
    # def __init__(self, filePath, pixmap, rect, arkaPlanRengi, cizgiRengi, parent=None, scene=None):
    def __init__(self, filePath, pos, rectf, pixmap, yaziRengi, arkaPlanRengi, pen, font, isEmbeded=False, parent=None):

        self.isEmbeded = isEmbeded
        self.filePathForSave = filePath
        if not isEmbeded:
            self.originalSourceFilePath = filePath
        else:
            self.originalSourceFilePath = None
            # bu zaten embeded ise, file open veya importtan cagriliyor demek,
            # dolayisi ile, ordan direkt set edilsin.

        if os.path.isfile(filePath):
            self.filePathForDraw = filePath
        else:
            self.filePathForDraw = ':icons/warning.png'

        # self.pixmap = QPixmapCache.find(self.filePathForDraw, self.pixmap)
        # self.pixmap = QPixmapCache.find(self.filePathForDraw)

        if pixmap:
            self.pixmap = pixmap
        else:
            # self.pixmap = QPixmap()
            self.pixmap = QPixmap(self.filePathForDraw)

        # else:
        #     self.pixmap = QPixmap()
        #     if not QPixmapCache.find(self.filePathForDraw, self.pixmap):
        #         self.pixmap = QPixmap(self.filePathForDraw)
        #         QPixmapCache.insert(self.filePathForDraw, self.pixmap)
        # print(QPixmapCache.cacheLimit())

        if not rectf:
            rectf = QRectF(self.pixmap.rect())

        super(Image, self).__init__(pos, rectf, yaziRengi, arkaPlanRengi, pen, font, parent)
        # self.setCacheMode(Image.CacheMode.DeviceCoordinateCache)
        # self.setCacheMode(Image.CacheMode.ItemCoordinateCache)
        self.orjinal_boyut = self.pixmap.size()
        self.bilgi_olcek = "1 - 1"

        self.setFlags(BaseItem.GraphicsItemFlag.ItemIsMovable |
                      BaseItem.GraphicsItemFlag.ItemIsSelectable)

        self.imageOpacity = 1.0
        self.isMirrorX = False
        self.isMirrorY = False

        # self.ItemClipsToShape()
        # self.opaqueArea()

        self._isCropping = False
        self.crop_first_point = QPointF()
        self.cropRectF = QRectF()

    # ---------------------------------------------------------------------
    def type(self):
        # Enable the use of qgraphicsitem_cast with this item.
        return Image.Type

    # ---------------------------------------------------------------------
    def get_properties_for_save_binary(self):
        properties = {"type": self.type(),
                      "kim": self._kim,
                      "isEmbeded": self.isEmbeded,
                      "filePath": self.filePathForSave,
                      "originalSourceFilePath": self.originalSourceFilePath,
                      "rect": self._rect,
                      "pos": self.pos(),
                      "rotation": self.rotation(),
                      "zValue": self.zValue(),
                      "yaziRengi": self.yaziRengi,
                      "arkaPlanRengi": self.arkaPlanRengi,
                      "pen": self._pen,
                      "font": self._font,
                      "imageOpacity": self.imageOpacity,
                      "yaziHiza": int(self.painterTextOption.alignment()),
                      "text": self.text(),
                      "isPinned": self.isPinned,
                      "isMirrorX": self.isMirrorX,
                      "isMirrorY": self.isMirrorY,
                      "command": self._command,
                      }
        return properties

    # ---------------------------------------------------------------------
    def mirror(self, x, y):
        if x:
            self.isMirrorX = not self.isMirrorX
        if y:
            self.isMirrorY = not self.isMirrorY
        self.reload_image_after_scale()

    # ---------------------------------------------------------------------
    def flipHorizontal(self, mposx):
        self.mirror(x=True, y=False)
        super(Image, self).flipHorizontal(mposx)

    # ---------------------------------------------------------------------
    def flipVertical(self, mposy):
        self.mirror(x=False, y=True)
        super(Image, self).flipVertical(mposy)

    # ---------------------------------------------------------------------
    def contextMenuEvent(self, event):
        # TODO: bu alttaki iki satir aktif idi, ve de ctrl harici sag tiklayinca
        # (base- text -path -group nesnelerinin contextMenuEventinde var)
        # mesela birden fazla nesne secili ve de gruplayacagız diyelim sag menu ile
        # ctrl basılı degil ise tikladigimiz secili kaliyor digerleri siliniyordu
        # uygunsuz bir kullanıcı deneyimi, niye yaptık acaba boyleyi hatırlayana kadar kalsın burda :)
        # if not event.modifiers() == Qt.KeyboardModifier.ControlModifier:
        #     self.scene().clearSelection()
        self.setSelected(True)

        # self.scene().parent().item_context_menu(self, event.screenPos())
        self.scene().parent().on_resim_sag_menu_about_to_show(self)
        self.scene().parent().resimSagMenu.popup(event.screenPos())

        event.accept()

    # ---------------------------------------------------------------------
    def mouseDoubleClickEvent(self, event):

        if event.modifiers() == Qt.KeyboardModifier.ControlModifier:
            self.orjinal_boyuta_dondur()
            return  # QGraphicsItem.mouseDoubleClickEvent(self, event)

        if event.button() == Qt.MouseButton.LeftButton:
            textItem = Text(event.scenePos(), self.yaziRengi, self.arkaPlanRengi, self.pen(), self.font())
            textItem.set_document_url(self.scene().tempDirPath)
            textItem.textItemFocusedOut.connect(self.scene().is_text_item_empty)
            self.scene().parent().increase_zvalue(textItem)
            # textItem.textItemSelectedChanged.connect(self.textItemSelected)
            # self.addItem(textItem)
            self.scene().undoRedo.undoableAddItem(self.scene().undoStack, description=self.scene().tr("add text"),
                                                  scene=self.scene(),
                                                  item=textItem)
            textItem.setFocus()
            yeniPos = self.mapFromScene(textItem.scenePos())
            self.scene().undoRedo.undoableParent(self.scene().undoStack, self.scene().tr("_parent"), textItem, self,
                                                 yeniPos)

        # super(Image, self).mouseDoubleClickEvent(event)

    # ---------------------------------------------------------------------
    def mousePressEvent(self, event):
        if self._isCropping:
            self.crop_first_point = event.pos()
        super(Image, self).mousePressEvent(event)

    # ---------------------------------------------------------------------
    def mouseMoveEvent(self, event):
        # print("move")
        # if event.button() == Qt.MiddleButton:
        #     print("middle")

        if self._isCropping:
            self.cropRectF = QRectF(self.crop_first_point, event.pos()).normalized()
            self.update(self.cropRectF)
            # self.update(self.cropRectF.adjusted(-10, -10, 10, 10))
            # super(Image, self).mouseMoveEvent(event)

            # print(QRectF(self.crop_first_point, self.crop_release_point))
            return

        if self._resizing:
            # cunku px=1 , py=1 olarak basliyor cizmeye. burda sifirliyoruz eger ilk sahneye ekleme cizimi ise.
            if not self._eskiRectBeforeResize.size() == QSizeF(1, 1):
                px = self._fixedResizePoint.x()
                py = self._fixedResizePoint.y()
            else:
                px = py = 0
            # mx = event.scenePos().x()
            # my = event.scenePos().y()
            mx = event.pos().x()
            my = event.pos().y()
            topLeft = QPointF(min(px, mx), min(py, my))  # top-left corner (x,y)
            bottomRight = QPointF(max(px, mx), max(py, my))  # bottom-right corner (x,y)
            # size = QSizeF(fabs(mx - px), fabs(my - py))
            # rect = QRectF(topLeft, size)

            rect = QRectF(topLeft, bottomRight)

            c = self._rect.center()

            # Alt Key - to resize around center.
            # if event.modifiers() & Qt.AltModifier:
            if event.modifiers() == Qt.KeyboardModifier.AltModifier:
                rect.moveCenter(c)

            # ---------------------------------------------------------------------
            #  Ctrl Key - to keep aspect ratio while resizing.
            if event.modifiers() == Qt.KeyboardModifier.ControlModifier:
                tl = self._rect.topLeft()
                tr = self._rect.topRight()
                br = self._rect.bottomRight()
                bl = self._rect.bottomLeft()
                c = self._rect.center()

                yeniSize = rect.size()
                # eskiSize = self._rect.size()
                pixMapOriginalSize = self.pixmap.rect().size()
                # eskiSize.scale(yeniSize, Qt.KeepAspectRatio)
                pixMapOriginalSize.scale(yeniSize.height(), yeniSize.height(),
                                         Qt.AspectRatioMode.KeepAspectRatioByExpanding)
                # eskiSize.scale(yeniSize.height(), yeniSize.height(), Qt.KeepAspectRatio)

                # if not eskiSize.isNull():
                if not pixMapOriginalSize.isEmpty():
                    self.yedekSize = QSizeF(pixMapOriginalSize)

                else:
                    pixMapOriginalSize = QSizeF(self.yedekSize)

                rect.setSize(QSizeF(pixMapOriginalSize))
                h = rect.height()
                w = rect.width()

                if self._resizingFrom == 1:
                    rect.moveTopLeft(QPointF(br.x() - w, br.y() - h))

                if self._resizingFrom == 2:
                    rect.moveTopRight(QPointF(bl.x() + w, bl.y() - h))

                if self._resizingFrom == 3:
                    rect.moveBottomRight(QPointF(tl.x() + w, tl.y() + h))

                elif self._resizingFrom == 4:
                    rect.moveBottomLeft(QPointF(tr.x() - w, tr.y() + h))

                # Alt Key - to resize around center
                if event.modifiers() == Qt.KeyboardModifier.AltModifier:
                    rect.moveCenter(c)

            # ---------------------------------------------------------------------
            # Shift Key - to make square (equals width and height)
            if event.modifiers() == Qt.KeyboardModifier.ShiftModifier:
                height = my - py
                if self._resizingFrom == 1:
                    rect.setCoords(px + height, py + height, px, py)
                    rect = rect.normalized()

                elif self._resizingFrom == 2:
                    rect.setCoords(px, py + height, px - height, py)
                    rect = rect.normalized()

                elif self._resizingFrom == 3:
                    rect.setCoords(px, py, px + height, py + height)
                    rect = rect.normalized()

                elif self._resizingFrom == 4:
                    rect.setCoords(px - height, py, px, py + height)
                    rect = rect.normalized()

                # Alt Key - to resize around center
                if event.modifiers() == Qt.KeyboardModifier.AltModifier:
                    rect.moveCenter(c)

            self.setRect(self.mapRectFromItem(self, rect))  # mouse release eventten gonderiyoruz undoya
            self.update_resize_handles()
            self.scene().parent().change_transform_box_values(self)

            self.bilgi_olcek = f"{rect.width() / self.orjinal_boyut.width():.2f} " \
                               f"- {rect.height() / self.orjinal_boyut.height():.2f}"
            self.scene().parent().log(txt=self.bilgi_olcek, msecs=500)
            # self.setPos(x, y)
            # self.update_painter_text_rect()

        # event.accept()
        else:
            super(Image, self).mouseMoveEvent(event)

    # ---------------------------------------------------------------------
    def orjinal_boyuta_dondur(self):

        yeniRect = QRectF(self._rect)
        yeniRect.setSize(self.orjinal_boyut)

        self.scene().undoRedo.undoableResizeBaseItem(self.scene().undoStack,
                                                     "resize to original",
                                                     self,
                                                     # yeniRect=self._rect,
                                                     yeniRect=yeniRect,
                                                     eskiRect=QRectF(self._rect),
                                                     eskiPos=self.pos())

        self.update_resize_handles()
        self.scene().parent().change_transform_box_values(self)
        self.update_painter_text_rect()
        self.scene().unite_with_scene_rect(self.sceneBoundingRect())

    # ---------------------------------------------------------------------
    def reload_image_after_scale(self):
        # self.setTransformOriginPoint(self.boundingRect().center())
        if not os.path.isfile(self.filePathForSave):
            self.filePathForDraw = ':icons/warning.png'
        else:
            self.filePathForDraw = self.filePathForSave

        # pixmap = QPixmapCache.find(self.filePathForDraw)
        # if not pixmap:
        #     pixmap = QPixmap(self.filePathForDraw)
        #     QPixmapCache.insert(self.filePathForDraw, pixmap)

        # pixmap = QPixmap()
        # if not QPixmapCache.find(self.filePathForDraw, pixmap):
        #     QPixmapCache.insert(self.filePathForDraw, pixmap)

        pixmap = QPixmap(self.filePathForDraw)

        size = self._rect.size().toSize()
        self.pixmap = pixmap.scaled(size, Qt.AspectRatioMode.KeepAspectRatio)

        if self.isMirrorX:
            self.pixmap = self.pixmap.transformed(QTransform().scale(-1, 1))
        if self.isMirrorY:
            self.pixmap = self.pixmap.transformed(QTransform().scale(1, -1))

        self.bilgi_olcek = f"{size.width() / self.orjinal_boyut.width():.2f} " \
                           f"- {size.height() / self.orjinal_boyut.height():.2f}"

    # ---------------------------------------------------------------------
    def finish_crop(self):
        rectf = self.cropRectF
        # croppedPixmap = self.pixmap.copy(rect)
        # secim disari tasarsa
        rectf = rectf.intersected(self._rect)
        if not rectf.isEmpty():
            srectf = self.mapRectToScene(rectf)
            pixMap = QPixmap(srectf.size().toSize())
            # son bir update, yoksa bazen crop rect cizgisi kaliyor render icinde
            self.update(rectf)
            painter = QPainter()

            if not painter.begin(pixMap):
                # print("hata")
                self._isCropping = False
                self.cropRectF = QRectF()
                return

            painter.setRenderHint(QPainter.RenderHint.Antialiasing)
            self.scene().render(painter, QRectF(), srectf)

            painter.end()

            # image = QImage(self.pixMap.copy(rect))
            fileName = os.path.splitext(os.path.basename(self.filePathForSave))[0]
            imageSavePath = self.scene().get_unique_path_for_embeded_image("cropped_{}.jpg".format(fileName))
            pixMap.save(imageSavePath)

            pos = self.scenePos()

            pos.setX(pos.x() + 5)
            pos.setY(pos.y() + 5)

            imageItem = Image(imageSavePath, pos, rectf, pixMap, self.yaziRengi, self.arkaPlanRengi,
                              self._pen, self.font(), isEmbeded=True)

            self.scene().parent().increase_zvalue(imageItem)
            self.scene().undoRedo.undoableAddItem(self.scene().undoStack, "add new cropped image", self.scene(),
                                                  imageItem)
            imageItem.reload_image_after_scale()
            self.scene().unite_with_scene_rect(imageItem.sceneBoundingRect())
            self.scene().parent().setCursor(Qt.CursorShape.ArrowCursor)

        self._isCropping = False
        self.cropRectF = QRectF()
        self.scene().parent().setCursor(Qt.CursorShape.ArrowCursor)

    # ---------------------------------------------------------------------
    def mouseReleaseEvent(self, event):
        if self._isCropping:
            self.finish_crop()

        super(Image, self).mouseReleaseEvent(event)

        self.reload_image_after_scale()

    # ---------------------------------------------------------------------
    def hoverEnterEvent(self, event):
        # event propagationdan dolayi bos da olsa burda implement etmek lazim
        # mousePress,release,move,doubleclick de bu mantıkta...
        # baska yerden baslayip mousepress ile , mousemove baska, mouseReleas baska widgetta gibi
        # icinden cikilmaz durumlara sebep olabiliyor.
        # ayrica override edince accept() de edilmis oluyor mouseeventler
        super(Image, self).hoverEnterEvent(event)

    # ---------------------------------------------------------------------
    def hoverMoveEvent(self, event):
        # self.sagUstKare.hide()

        # cursor = self.scene().parent().cursor()
        if self._isCropping:
            self.scene().parent().setCursor(Qt.CursorShape.CrossCursor, gecici_mi=True)
        else:
            super(Image, self).hoverMoveEvent(event)

    # ---------------------------------------------------------------------
    def hoverLeaveEvent(self, event):
        if self._isCropping:
            self.finish_crop()
        super(Image, self).hoverLeaveEvent(event)

    # ---------------------------------------------------------------------
    def itemChange(self, change, value):
        # if change == self.ItemPositionChange:
        # if change == self.ItemSelectedChange:
        # if change == self.ItemSelectedHasChanged:
        if change == BaseItem.GraphicsItemChange.ItemSelectedChange:
            if value:
                self.scene().parent().item_selected(self)
            else:  # yani deselected
                self.scene().parent().item_deselected(self)

        # return QGraphicsRectItem.itemChange(self, change, value)
        return value

    # ---------------------------------------------------------------------
    def wheelEvent(self, event):
        # factor = 1.41 ** (event.delta() / 240.0)
        # self.scale(factor, factor)

        if self.ustGrup:
            return BaseItem.wheelEvent(self, event)

        toplam = event.modifiers().value

        # ctrl = int(Qt.ControlModifier)
        shift = Qt.KeyboardModifier.ShiftModifier.value
        alt = Qt.KeyboardModifier.AltModifier.value

        ctrlAlt = Qt.KeyboardModifier.ControlModifier.value + Qt.KeyboardModifier.AltModifier.value
        ctrlShift = Qt.KeyboardModifier.ControlModifier.value + Qt.KeyboardModifier.ShiftModifier.value
        altShift = Qt.KeyboardModifier.AltModifier.value + Qt.KeyboardModifier.ShiftModifier.value
        ctrlAltShift = Qt.KeyboardModifier.ControlModifier.value + Qt.KeyboardModifier.AltModifier.value + Qt.KeyboardModifier.ShiftModifier.value

        # if event.modifiers() & Qt.ControlModifier:
        if toplam == ctrlShift:
            # self.scaleItem(event.delta())
            self.scaleItemByResizing(event.delta())

        # elif event.modifiers() & Qt.ShiftModifier:
        elif toplam == shift:
            self.changeFontSizeF(event.delta())

        # elif event.modifiers() & Qt.AltModifier:
        elif toplam == alt:
            # self.changeImageAlpha(event.delta())
            self.changeBackgroundColorAlpha(event.delta())

        elif toplam == ctrlAlt:
            # self.changeLineColorAlpha(event.delta())
            self.changeTextColorAlpha(event.delta())

        elif toplam == ctrlAltShift:
            self.rotateItem(event.delta())

        elif toplam == altShift:
            # self.changeTextBackgroundColorAlpha(event.delta())
            # self.changeLineColorAlpha(event.delta())
            self.changeImageItemTextBackgroundColorAlpha(event.delta())

        # elif toplam == ctrlAltShift:
        #     self.scaleItemByResizing(event.delta())

        else:
            super(Image, self).wheelEvent(event)

    # ---------------------------------------------------------------------
    def changeImageItemTextBackgroundColorAlpha(self, delta):

        col = shared.calculate_alpha(delta, QColor(self.arkaPlanRengi))

        if self.childItems():
            self.scene().undoStack.beginMacro("change image items' text background alpha")
            self.scene().undoRedo.undoableSetItemBackgroundColorAlpha(self.scene().undoStack,
                                                                      "_change image item's background alpha", self,
                                                                      col)
            for c in self.childItems():
                c.changeImageItemTextBackgroundColorAlpha(delta)
            self.scene().undoStack.endMacro()
        else:
            self.scene().undoRedo.undoableSetItemBackgroundColorAlpha(self.scene().undoStack,
                                                                      "change image item's background alpha", self, col)

    # ---------------------------------------------------------------------
    def changeBackgroundColorAlpha(self, delta):
        # self.changeImageAlpha(delta)
        if self.childItems():
            self.scene().undoStack.beginMacro("change items' background alpha")
            self.changeImageOpacity(delta)
            for c in self.childItems():
                c.changeBackgroundColorAlpha(delta)
            self.scene().undoStack.endMacro()
        else:
            self.changeImageOpacity(delta)

    # ---------------------------------------------------------------------
    def changeImageOpacity(self, delta):
        if delta > 0:
            if self.imageOpacity + 0.05 < 1:
                self.imageOpacity += 0.05
            else:
                self.imageOpacity = 1
        else:
            if self.imageOpacity - 0.05 > 0:
                self.imageOpacity -= 0.05
            else:
                self.imageOpacity = 0

        self.update()

        # self.pixmap.setAlphaChannel()

        # self.setBrush(self.arkaPlanRengi)
        # self.update()
        self.scene().undoRedo.undoableSetImageOpacity(self.scene().undoStack, "change image opacity", self,
                                                      self.imageOpacity)

    # ---------------------------------------------------------------------
    def changeFontSizeF(self, delta):

        # font = self.font()
        size = self.fontPointSizeF()

        if delta > 0:
            # font.setPointSizeF(size + 1)
            size += 1

        else:
            if size > 10:
                # font.setPointSizeF(size - 1)
                size -= 1
            else:
                # undolari biriktermesin diye donuyoruz,
                # yoksa zaten ayni size de yeni bir undolu size komutu veriyor.
                return

        # self.setFont(font)
        if self.childItems():
            self.scene().undoStack.beginMacro("change text")
            self.scene().undoRedo.undoableSetFontSizeF(self.scene().undoStack, "change text size", self, size)
            for c in self.childItems():
                c.changeFontSizeF(delta)
            self.scene().undoStack.endMacro()
        else:
            self.scene().undoRedo.undoableSetFontSizeF(self.scene().undoStack, "change text size", self, size)

    # ---------------------------------------------------------------------
    def update_painter_text_rect(self):
        # dummy method
        # basede var, ordaki hesaplamalari yapmasin
        pass

    # ---------------------------------------------------------------------
    def paint(self, painter, option, widget=None):

        painter.setClipRect(option.exposedRect)

        painter.setBrush(Qt.BrushStyle.NoBrush)
        # rect = self.boundingRect().toRect()
        rect = self._rect.toRect()
        painter.setOpacity(self.imageOpacity)
        painter.drawPixmap(rect, self.pixmap)
        painter.setOpacity(1)

        if option.state & QStyle.StateFlag.State_Selected or self.cosmeticSelect:

            if self.isActiveItem:
                selectionPenBottom = self.selectionPenBottomIfAlsoActiveItem
                if not self.isPinned:
                    painter.setPen(Qt.PenStyle.NoPen)
                    painter.setBrush(self.handleBrush)
                    painter.drawRect(self.topLeftHandle)
                    painter.drawRect(self.topRightHandle)
                    painter.drawRect(self.bottomRightHandle)
                    painter.drawRect(self.bottomLeftHandle)
            else:
                selectionPenBottom = self.selectionPenBottom

            painter.setBrush(Qt.BrushStyle.NoBrush)

            painter.setPen(selectionPenBottom)
            painter.drawRect(rect)

            if self._resizing:
                oran = 1 / self.scene().views()[0].transform().m11()
                font = painter.font()
                font.setPointSizeF(font.pointSizeF() * oran)
                painter.setFont(font)
                painter.drawText(rect.topLeft().x(), rect.topLeft().y() + 10, self.bilgi_olcek)

            # painter.setPen(self.selectionPenTop)
            # painter.drawRect(self._rect)

            ########################################################################
            # !!! simdilik iptal, gorsel fazlalik olusturmakta !!!
            ########################################################################
            # if not self.isPinned and self.isActiveItem:
            # painter.setPen(Qt.PenStyle.NoPen)
            # painter.setBrush(self.handleBrush)
            # painter.drawRect(self.topLeftHandle)
            # painter.drawRect(self.topRightHandle)
            # painter.drawRect(self.bottomRightHandle)
            # painter.drawRect(self.bottomLeftHandle)
            ########################################################################
        if self._isCropping:
            # painter.setBrush(Qt.green)
            painter.setPen(self.yaziRengi)
            # painter.setOpacity(.35)
            painter.drawRect(self.cropRectF)

        # if option.state & QStyle.StateFlag.State_MouseOver:
        #     painter.setBrush(Qt.BrushStyle.NoBrush)
        #     painter.setPen(self.selectionPenBottom)
        #     painter.drawRect(self._rect)

        # # # # # # debug start - pos() # # # # #
        # p = self.pos()
        # s = self.scenePos()
        # painter.drawText(self._rect, "{0:.2f},  {1:.2f} pos \n{2:.2f},  {3:.2f} spos".format(p.x(), p.y(), s.x(), s.y()))
        # # # t = self.transformOriginPoint()
        # # # painter.drawRect(t.x()-12, t.y()-12,24,24)
        # mapped = self.mapToScene(self._rect.topLeft())
        # painter.drawText(self._rect.x(), self._rect.y(), "{0:.2f}  {1:.2f} map".format(mapped.x(), mapped.y()))
        # painter.drawEllipse(self.scenePos(), 10, 10)
        # painter.setPen(Qt.blue)
        # painter.drawEllipse(self.mapFromScene(self.pos()), 10, 10)
        # r = self.textItem.boundingRect()
        # r = self.mapRectFromItem(self.textItem, r)
        # painter.drawRect(r)
        # painter.drawText(self._rect.center(), "{0:f}  {1:f}".format(self.sceneWidth(), self.sceneHeight()))
        # painter.setPen(QPen(Qt.red,17))
        # painter.drawPoint(self._rect.center())
        # painter.setPen(QPen(Qt.green,12))
        # painter.drawPoint(self.mapFromScene(self.sceneBoundingRect().center()))
        # painter.setPen(QPen(Qt.blue,8))
        # painter.drawPoint(self.sceneBoundingRect().center())
        # painter.drawRect(self.sceneBoundingRect())
        # # # # # # debug end - pos() # # # # #

    # ---------------------------------------------------------------------
    def html_dive_cevir(self, html_klasor_kayit_adres, resim_kopyalaniyor_mu):
        # resim_adres= f'<img src="{item.filePathForSave}" style="object-fit: fill;"></img>'
        resim_adi = os.path.basename(self.filePathForSave)
        resim_adres = self.filePathForSave
        if not html_klasor_kayit_adres:  # def dosyasi icine kaydediliyor
            if self.isEmbeded:
                resim_adres = os.path.join("images", resim_adi)
        else:  # dosya html olarak baska bir yere kaydediliyor
            # kopyalanmazsa da, zaten embed olmayan resim normal hddeki adresten yuklenecektir.
            if resim_kopyalaniyor_mu:
                # iptal: if not self.isEmbeded:  # embed ise zaten tmp klasorden hedef klasore baska metodta kopylanıyor hepsi.
                # embed veya degil, yukarda bahsettigi gibi resim kopyalansa da adresi de guncellemek lazim.
                resim_adres = os.path.join(html_klasor_kayit_adres, "images", resim_adi)

        img_str = f'<img src="{resim_adres}" style="width:100%; height:100%;"></img>'

        w = self._rect.width()
        h = self._rect.height()

        c = self.sceneBoundingRect().center()

        xr = c.x() - w / 2
        yr = c.y() - h / 2
        xs = self.scene().sceneRect().x()
        ys = self.scene().sceneRect().y()
        x = xr - xs
        y = yr - ys

        bicimSozluk = self.ver_karakter_bicimi()
        bold = "font-weight:bold;" if bicimSozluk["b"] else ""
        italic = "font-style:italic;" if bicimSozluk["i"] else ""
        underline = "underline" if bicimSozluk["u"] else ""
        strikeOut = "line-through" if bicimSozluk["s"] else ""
        overline = "overline" if bicimSozluk["o"] else ""
        bicimler1 = bold + italic
        if any((underline, strikeOut, overline)):
            bicimler2 = f"text-decoration: {underline} {strikeOut} {overline};"
        else:
            bicimler2 = ""

        renk_arkaPlan = f"({self.arkaPlanRengi.red()},{self.arkaPlanRengi.green()},{self.arkaPlanRengi.blue()},{self.arkaPlanRengi.alpha() / 255})"
        renk_yazi = f"({self.yaziRengi.red()},{self.yaziRengi.green()},{self.yaziRengi.blue()},{self.yaziRengi.alpha() / 255})"

        div_str = f"""
                    <div style="
                     background:rgba{renk_arkaPlan};
                     color:rgba{renk_yazi};
                     font-size:{self.fontPointSizeF()}pt; 
                     font-family:{self.font().family()};
                     {bicimler1}
                     {bicimler2}
                     position:absolute;
                     z-index:{int(self.zValue() * 10) if self.zValue() else 0};
                     width:{self._rect.width()}px;
                     height:{self._rect.height()}px;
                     top:{y}px;
                     left:{x}px;
                     opacity:{self.imageOpacity};
                     transform-box: fill-box;
                     transform-origin: center;
                     transform: rotate({self.rotation()}deg);
                     " id="{self._kim}">{img_str}</div>\n"""

        # /*background-image:url('{resim_adres}');*/
        return div_str



# -*- coding: utf-8 -*-
# .

__project_name__ = 'Defter'
__author__ = 'Erdinç Yılmaz'
__date__ = '3/28/16'

from PySide6.QtGui import QPen, QBrush, QPainterPath, QPainterPathStroker, QColor, QTextOption
from PySide6.QtWidgets import QGraphicsItem, QStyle
from PySide6.QtCore import QRectF, Qt, QSizeF, QPointF, Slot
from ..nesneler.tempTextItem import TempTextItem
from .. import shared


########################################################################
class BaseItem(QGraphicsItem):
    Type = shared.BASE_ITEM_TYPE

    # ---------------------------------------------------------------------
    def __init__(self, pos, rect, yaziRengi, arkaPlanRengi, pen, font, parent=None):
        super(BaseItem, self).__init__(parent)
        # QGraphicsItem.__init__(self, parent,scene)

        self._kim = shared.kim(kac_basamak=16)

        self.setPos(pos)
        self._rect = rect

        self.setAcceptHoverEvents(True)

        self._eskiRectBeforeResize = None
        self._eskiPosBeforeResize = None

        self.secili_nesne_kalem_kalinligi = 0

        # resim nesnesinde paint icinde kullaniyoruz,
        # o yuzden nesneye tiklanmamisken de lazim oluyor.
        self._resizing = False
        self.handleSize = 10
        self.resizeHandleSize = QSizeF(self.handleSize, self.handleSize)
        self.sifirNoktasi = QPointF(0, 0)

        self.setFlags(QGraphicsItem.GraphicsItemFlag.ItemIsSelectable |
                      QGraphicsItem.GraphicsItemFlag.ItemIsMovable |
                      QGraphicsItem.GraphicsItemFlag.ItemIsFocusable)

        # self.setFiltersChildEvents(True)
        # self.setHandlesChildEvents(True)

        self._pen = pen
        self._brush = QBrush()
        self._font = font

        # self.boundingRectTasmaDegeri = max(self.handleSize, self._pen.widthF() / 2)
        self.boundingRectTasmaDegeri = ((self._pen.widthF() / 2) + (self.handleSize/2))
        self._boundingRect = QRectF()

        self.update_resize_handles()

        self.painterTextOption = QTextOption()
        self.painterTextOption.setWrapMode(QTextOption.WrapMode.WrapAtWordBoundaryOrAnywhere)
        self.painterTextOption.setAlignment(Qt.AlignmentFlag.AlignCenter)

        self.painterTextRect = QRectF(self._rect)

        self.activeItemLineColor = shared.activeItemLineColor
        self.cizgiRengi = pen.color()
        self.setCizgiRengi(self.cizgiRengi)  # also sets self._pen
        self.yaziRengi = yaziRengi
        self.setYaziRengi(yaziRengi)
        self.setArkaPlanRengi(arkaPlanRengi)  # also sets self._brush

        self.cosmeticSelect = False
        self.isActiveItem = False
        self._isPinned = False
        self.ustGrup = None

        self._text = ""
        self._command = {}

        self.tempTextItem = None
        self.tempEskiText = ""

        self.textPadding = 20

        # bagli_oklarin_dxdysi_ve_okun_hangi_noktasi_bagli_sozluk
        self.oklar_dxdy_nokta = {}

    # ---------------------------------------------------------------------
    def type(self):
        # Enable the use of qgraphicsitem_cast with this item.
        return BaseItem.Type

    # ---------------------------------------------------------------------
    @property
    def isPinned(self):
        return self._isPinned

    # ---------------------------------------------------------------------
    @isPinned.setter
    def isPinned(self, value):
        if value:
            self.setFlags(QGraphicsItem.GraphicsItemFlag.ItemIsSelectable
                          # | self.ItemIsMovable
                          #  | item.ItemIsFocusable
                          )

        else:
            self.setFlags(QGraphicsItem.GraphicsItemFlag.ItemIsSelectable
                          | QGraphicsItem.GraphicsItemFlag.ItemIsMovable
                          | QGraphicsItem.GraphicsItemFlag.ItemIsFocusable)
        self._isPinned = value

    # ---------------------------------------------------------------------
    def ok_ekle(self, ok, scenepPos, okunHangiNoktasi):

        # self.oklar_dxdy_nokta.append((ok, self.mapFromScene(scenePos)))
        sceneCenter = self.sceneCenter()
        dx = sceneCenter.x() - scenepPos.x()
        dy = sceneCenter.y() - scenepPos.y()

        # self.oklar_dxdy_nokta[ok._kim] = (dx, dy, nokta)
        self.oklar_dxdy_nokta[ok] = (dx, dy, okunHangiNoktasi)

        ok.baglanmis_nesneler[self._kim] = okunHangiNoktasi

        # self.setText(" ".join(a._kim for a in self.oklar_dxdy_nokta.keys()))

    # ---------------------------------------------------------------------
    def ok_sil(self, ok):
        del self.oklar_dxdy_nokta[ok]
        del ok.baglanmis_nesneler[self._kim]
        # self.setText(" ".join(a._kim for a in self.oklar_dxdy_nokta.keys()))

    # ---------------------------------------------------------------------
    def ok_guncelle(self):
        if self.oklar_dxdy_nokta:
            sceneCenter = self.sceneCenter()
            scx = sceneCenter.x()
            scy = sceneCenter.y()
            for ok, dxdy_nokta in self.oklar_dxdy_nokta.items():
                if dxdy_nokta[2] == 1:
                    ok.temp_prepend(QPointF(scx - dxdy_nokta[0], scy - dxdy_nokta[1]))
                elif dxdy_nokta[2] == 2:
                    ok.temp_append(QPointF(scx - dxdy_nokta[0], scy - dxdy_nokta[1]))
                if ok._text:
                    ok.update_painter_text_rect()

    # ---------------------------------------------------------------------
    def get_properties_for_save_binary(self):
        properties = {"type": self.type(),
                      "kim": self._kim,
                      "rect": self._rect,
                      "pos": self.pos(),
                      "rotation": self.rotation(),
                      "zValue": self.zValue(),
                      "yaziRengi": self.yaziRengi,
                      "arkaPlanRengi": self.arkaPlanRengi,
                      "pen": self._pen,
                      "font": self._font,
                      "yaziHiza": int(self.painterTextOption.alignment()),
                      "text": self.text(),
                      "isPinned": self.isPinned,
                      "command": self._command,
                      }
        return properties

    # ---------------------------------------------------------------------
    def varsaEnUsttekiGrubuGetir(self):
        parentItem = self.parentItem()
        while parentItem:
            if parentItem.type() == shared.GROUP_ITEM_TYPE:
                return parentItem
            parentItem = parentItem.parentItem()
        return None

    # ---------------------------------------------------------------------
    def sceneCenter(self):
        if self.parentItem():
            return self.mapToParent(self._rect.center())
        return self.mapToScene(self._rect.center())

    # ---------------------------------------------------------------------
    def sceneWidth(self):
        return self.sceneRight() - self.sceneLeft()

    # ---------------------------------------------------------------------
    def sceneHeight(self):
        return self.sceneBottom() - self.sceneTop()

    # ---------------------------------------------------------------------
    def sceneRight(self):
        # 4 koseyi de dikkate aliyoruz, mesela sadece sagdakileri degil,
        # cunku nesneyi saga veya sola dogru cizebiliyoruz. Ya da saga veya sola dogru boyutunu degistirebiliyoruz.

        r = self.mapRectToScene(self._rect)
        return max(r.topLeft().x(),
                   r.topRight().x(),
                   r.bottomRight().x(),
                   r.bottomLeft().x())

    # ---------------------------------------------------------------------
    def sceneLeft(self):
        r = self.mapRectToScene(self._rect)
        return min(r.topLeft().x(),
                   r.topRight().x(),
                   r.bottomRight().x(),
                   r.bottomLeft().x())

    # ---------------------------------------------------------------------
    def sceneTop(self):
        r = self.mapRectToScene(self._rect)
        return min(r.topLeft().y(),
                   r.topRight().y(),
                   r.bottomRight().y(),
                   r.bottomLeft().y())

    # ---------------------------------------------------------------------
    def sceneBottom(self):
        r = self.mapRectToScene(self._rect)
        return max(r.topLeft().y(),
                   r.topRight().y(),
                   r.bottomRight().y(),
                   r.bottomLeft().y())

    # ---------------------------------------------------------------------
    def setSceneLeft(self, left):
        self.setX(left + self.scenePos().x() - self.sceneLeft())

    # ---------------------------------------------------------------------
    def setSceneRight(self, right):
        self.setX(right + self.scenePos().x() - self.sceneRight())

    # ---------------------------------------------------------------------
    def setSceneTop(self, top):
        self.setY(top + self.scenePos().y() - self.sceneTop())

    # ---------------------------------------------------------------------
    def setSceneBottom(self, bottom):
        self.setY(bottom + self.scenePos().y() - self.sceneBottom())

    # ---------------------------------------------------------------------
    def setHorizontalCenter(self, hcenter):
        self.setY(hcenter + self.scenePos().y() - self.sceneCenter().y())

    # ---------------------------------------------------------------------
    def setVerticalCenter(self, vcenter):
        self.setX(vcenter + self.scenePos().x() - self.sceneCenter().x())

    # # ---------------------------------------------------------------------
    # def setVerticalCenterWhenParented(self, vcenter):
    #     self.setX(vcenter + self.scenePos().x() - self.sceneCenter().x() - self.parentItem().pos().x())

    # ---------------------------------------------------------------------
    def flipHorizontal(self, mposx):
        rc = self.mapToScene(self._rect.center())
        diff = mposx - rc.x()
        ipos = self.scenePos()
        ipos.setX(ipos.x() + 2 * diff)

        if self.parentItem():
            # eger self parented ise, parented degilmis gibi dusunup
            # sahne pozisyonlari ile mposxe gore yeni mirror pos unu bulup
            # buldugumuzdan sahne pos unu cıkarıp set ediyoruz.
            # bu methodu grup cagirdiginda, donusunde grup icinde bounding rect
            # tekrar hesaplandigi icin, Hamd Olsun islem tamam oluyor.
            # (ayrica grup rotation sifir degilse gecici olarak sifir yapiliyor.
            # burdaki islemler sifir rotationda oluyor. sonra bitince 360- origRot
            # ile grup flipped rotationa donduruluyor)
            # dolayısı ile burda grubun pos u ile ilgili baska bir isleme gerek kalmiyor.
            # ya da grup icersinde. yani grubu icerige giydiriyoruz. islem bitince.

            # ipos = (ipos - self.parentItem().scenePos())
            # pass
            ipos = self.parentItem().mapFromScene(ipos)
            # ipos = self.parentItem().transform().map(ipos)

        self.setPos(ipos)
        if self.rotation():
            self.rotateWithOffset(360 - self.rotation())

        eskiRot = self.rotation()
        if eskiRot:
            self.rotateWithOffset(0)
        # eger item parent ise, icindekileri de flip etmek icin,
        # oncesinde yine item rotationu sifirladik, icindekileri flip edince, yine eski rotationa donduruyoruz.
        # bu bi anlamda recursive bir method, neden yukardaki if self.parentItem() i kullanmadik
        # az sonra cagirdigimizda yine bu fonksiyonu, yani child itemin rotation hesaplamalari icin,
        # cunku burda yapmak daha basit ve daha hizli, bu halde iken hesaplamalari c++ yapiyor, eger az once
        # bahsedilen yerde yapilirsa her item icin python yapacak.
        for c in self.childItems():
            c.flipHorizontal(self.sceneCenter().x())
        if eskiRot:
            self.rotateWithOffset(eskiRot)

    # ---------------------------------------------------------------------
    def flipVertical(self, mposy):
        rc = self.mapToScene(self._rect.center())
        diff = mposy - rc.y()
        ipos = self.scenePos()
        ipos.setY(ipos.y() + 2 * diff)

        if self.parentItem():
            # ipos = (ipos - self.parentItem().scenePos())
            # dikkat: (self.mapFromScene(ipos))  =/= (self.parentItem().mapFromScene(ipos))
            ipos = self.parentItem().mapFromScene(ipos)

        self.setPos(ipos)
        if self.rotation():
            self.rotateWithOffset(360 - self.rotation())

        eskiRot = self.rotation()
        if eskiRot:
            self.rotateWithOffset(0)
        for c in self.childItems():
            c.flipVertical(self.sceneCenter().y())
        if eskiRot:
            self.rotateWithOffset(eskiRot)

    # ---------------------------------------------------------------------
    def update_resize_handles(self):
        self.prepareGeometryChange()

        if self.scene():
            resizeHandleSize = self.resizeHandleSize / self.scene().views()[0].transform().m11()
        else:
            resizeHandleSize = self.resizeHandleSize

        # bunlarin yeri onemli degil, QSize(0,0) da olur, asagida yerlerine kaydiriliorlar
        self.topLeftHandle = QRectF(self.sifirNoktasi, resizeHandleSize)
        self.topRightHandle = QRectF(self.sifirNoktasi, resizeHandleSize)
        self.bottomRightHandle = QRectF(self.sifirNoktasi, resizeHandleSize)
        self.bottomLeftHandle = QRectF(self.sifirNoktasi, resizeHandleSize)

        # self.topLeftHandle.moveTopLeft(self._rect.topLeft())
        # self.topRightHandle.moveTopRight(self._rect.topRight())
        # self.bottomRightHandle.moveBottomRight(self._rect.bottomRight())
        # self.bottomLeftHandle.moveBottomLeft(self._rect.bottomLeft())

        self.topLeftHandle.moveCenter(self._rect.topLeft())
        self.topRightHandle.moveCenter(self._rect.topRight())
        self.bottomRightHandle.moveCenter(self._rect.bottomRight())
        self.bottomLeftHandle.moveCenter(self._rect.bottomLeft())

    # ---------------------------------------------------------------------
    def hide_resize_handles(self):
        self.isPinned = True

    # ---------------------------------------------------------------------
    def show_resize_handles(self):
        self.isPinned = False

    # ---------------------------------------------------------------------
    def setRect(self, rect):
        if self._rect == rect:
            return
        self.prepareGeometryChange()
        self._rect = rect
        self._boundingRect = QRectF()
        self.update()

    # ---------------------------------------------------------------------
    def rect(self):
        return self._rect

    # ---------------------------------------------------------------------
    def setText(self, text):
        self._text = text
        self.update()

    # ---------------------------------------------------------------------
    def text(self):
        return self._text

    # ---------------------------------------------------------------------
    def setCommand(self, title, command):
        self._command["title"] = title
        self._command["command"] = command

    # ---------------------------------------------------------------------
    def command(self):
        return self._command

    # ---------------------------------------------------------------------
    def setFont(self, font):
        font.setBold(self._font.bold())
        font.setItalic(self._font.italic())
        font.setUnderline(self._font.underline())
        font.setStrikeOut(self._font.strikeOut())
        font.setOverline(self._font.overline())
        font.setPointSizeF(self._font.pointSizeF())
        self._font = font
        self.update()

    # ---------------------------------------------------------------------
    def font(self):
        return self._font

    # ---------------------------------------------------------------------
    def setFontPointSizeF(self, fontPointSizeF):
        self._font.setPointSizeF(fontPointSizeF)
        self.update()

    # ---------------------------------------------------------------------
    def fontPointSizeF(self):
        return self._font.pointSizeF()

    # ---------------------------------------------------------------------
    def ver_yazi_hizasi(self):
        return self.painterTextOption.alignment()

    # ---------------------------------------------------------------------
    def kur_yazi_hizasi(self, hizalama):
        self.painterTextOption.setAlignment(hizalama | Qt.AlignmentFlag.AlignVCenter)
        # self.painterTextOption.setAlignment(hizalama)
        self.update()

    # ---------------------------------------------------------------------
    def ver_karakter_bicimi(self):
        # return self._font.bold | self._font.italic | self._font.under | self._font.strikeout
        return {"b": self._font.bold(),
                "i": self._font.italic(),
                "u": self._font.underline(),
                "s": self._font.strikeOut(),
                "o": self._font.overline(),
                }

    # ---------------------------------------------------------------------
    def kur_karakter_bicimi(self, sozluk):
        self._font.setBold(sozluk["b"])
        self._font.setItalic(sozluk["i"])
        self._font.setUnderline(sozluk["u"])
        self._font.setStrikeOut(sozluk["s"])
        self._font.setOverline(sozluk["o"])
        self.update()

    # ---------------------------------------------------------------------
    def setPen(self, pen):

        if self._pen == pen:
            return
        self.prepareGeometryChange()
        self._pen = pen
        self._boundingRect = QRectF()
        self.update(self.boundingRect())

    # ---------------------------------------------------------------------
    def pen(self):
        return self._pen

    # ---------------------------------------------------------------------
    def setBrush(self, brush):

        if self._brush == brush:
            return
        self._brush = brush
        self.update()

    # ---------------------------------------------------------------------
    def brush(self):
        return self._brush

    # ---------------------------------------------------------------------
    def setCizgiRengi(self, col):
        # selectionLineBgColor = QColor(
        #     # 127,
        #                               0 if col.red() > 127 else 255,
        #                               0 if col.green() > 127 else 255,
        #                               0 if col.blue() > 127 else 255
        #                               )
        # selectionLineBgColor = QColor(
        #                               # 20 if max(col.value(),col.saturation()) < 185 else 200,
        #                               20 if col.value() < 215 else 220,
        #                               25 if col.value() < 128 else 220,
        #                               40 if col.value() < 185 else 255
        #                               )

        # selectionLineBgColor.setHsv()
        # selectionLineBgColor = QColor.fromHsl((col.hue() + 180) % 360,
        #                                       col.saturation(),
        #                                       255 - col.lightness())

        # selectionLineFgColor = QColor(
        #     255 if selectionLineBgColor.red() < 128 else 0,
        #     255 if selectionLineBgColor.green() < 128 else 0,
        #     255 if selectionLineBgColor.blue() < 128 else 0
        # )

        _selectionLineBgColor = QColor.fromHsv(col.hue(),
                                               0,
                                               20 if col.value() > 127 else 250)

        _activeItemLineColor = QColor(self.activeItemLineColor)
        if col.hue() > 300 or col.hue() < 30:
            _activeItemLineColor.setHsv((col.hue() + 150) % 360,
                                        self.activeItemLineColor.saturation(),
                                        self.activeItemLineColor.value())

        # selectionLineFgColor = QColor.fromHsv(col.hue(),
        #                                       0,
        #                                       20 if col.value() < 127 else 240)

        self._pen.setColor(col)
        self.cizgiRengi = col

        # drawing text after drawing rect does not apply alpha
        # we need to reconstruct the color with same values.
        # self.textPen = QPen(QColor().fromRgb(col.red(), col.green(), col.blue(), col.alpha()))

        self.selectionPenBottom = QPen(_selectionLineBgColor,
                                       self.secili_nesne_kalem_kalinligi,
                                       Qt.PenStyle.DashLine,
                                       Qt.PenCapStyle.RoundCap, Qt.PenJoinStyle.RoundJoin)
        self.selectionPenBottomIfAlsoActiveItem = QPen(_activeItemLineColor,
                                                       self.secili_nesne_kalem_kalinligi,
                                                       Qt.PenStyle.DashLine,
                                                       Qt.PenCapStyle.RoundCap, Qt.PenJoinStyle.RoundJoin)
        # self.selectionPenTop = QPen(col,
        #                             self.secili_nesne_kalem_kalinligi,
        #                             Qt.DashLine,
        #                             Qt.FlatCap, Qt.RoundJoin)

        # cizgi kalinligi zoomla artmasin istersek
        # self._pen.setCosmetic(True)
        # self.selectionPenBottom.setCosmetic(True)
        # self.selectionPenBottomIfAlsoActiveItem.setCosmetic(True)
        # self.selectionPenTop.setCosmetic(True)

        # self.handlePen = QPen(_selectionLineBgColor, self.secili_nesne_kalem_kalinligi, Qt.PenStyle.SolidLine)
        self.handlePen = QPen(_activeItemLineColor, self.secili_nesne_kalem_kalinligi, Qt.PenStyle.SolidLine)
        if col.value() > 245:
            self.handleBrush = shared.handleBrushKoyu
        else:
            self.handleBrush = shared.handleBrushAcik

        self.update()

    # ---------------------------------------------------------------------
    def setYaziRengi(self, col):

        # drawing text after drawing rect does not apply alpha
        # we need to reconstruct the color with same values.
        self.yaziRengi = col
        # self.textPen = QPen(QColor().fromRgb(col.red(), col.green(), col.blue(), col.alpha()))
        self.textPen = QPen(col)
        self.update()

    # ---------------------------------------------------------------------
    def setCizgiKalinligi(self, width):
        self._pen.setWidthF(width)
        self.textPen.setWidthF(width)
        # self.boundingRectTasmaDegeri = max(self.handleSize, width / 2)
        self.boundingRectTasmaDegeri = ((self._pen.widthF() / 2) + (self.handleSize/2))
        # self.selectionPenBottom.setWidthF(width)
        # self.selectionPenBottomIfAlsoActiveItem.setWidthF(width)
        # self.selectionPenTop.setWidthF(width)

        self.update()

    # ---------------------------------------------------------------------
    def setArkaPlanRengi(self, col):
        self.arkaPlanRengi = col
        self.setBrush(QBrush(col))

    # ---------------------------------------------------------------------
    def boundingRect(self):
        # if self._boundingRect.isNull():
        self._boundingRect = QRectF(self._rect)
        return self._boundingRect.adjusted(-self.boundingRectTasmaDegeri, -self.boundingRectTasmaDegeri,
                                           self.boundingRectTasmaDegeri, self.boundingRectTasmaDegeri)

        # path = QPainterPath()
        # path.addRect(self._rect)
        # # path.moveTo(0,0)
        # return path.boundingRect()

    # ---------------------------------------------------------------------
    def shape(self):
        path = QPainterPath()
        # path.addRect(self.rect)
        # path.addRect(self._rect)
        path.addRect(self.boundingRect())
        return self.qt_graphicsItem_shapeFromPath(path, self._pen)

    # ---------------------------------------------------------------------
    def qt_graphicsItem_shapeFromPath(self, path, pen):

        # We unfortunately need this hack as QPainterPathStroker will set a width of 1.0
        # if we pass a value of 0.0 to QPainterPathStroker.setWidth()
        penWidthZero = 0.00000001
        if path == QPainterPath():
            return path
        ps = QPainterPathStroker()
        ps.setCapStyle(pen.capStyle())

        if pen.widthF() <= 0:
            ps.setWidth(penWidthZero)
        else:
            ps.setWidth(pen.widthF())
        ps.setJoinStyle(pen.joinStyle())
        ps.setMiterLimit(pen.miterLimit())
        p = ps.createStroke(path)  # return type: QPainterPath
        p.addPath(path)
        return p

    # ---------------------------------------------------------------------
    # TODO: maybe: to: self.paint(): if option.state & QStyle.State_Selected:
    # const QRectF murect = painter->transform().mapRect(QRectF(0, 0, 1, 1));
    # if (qFuzzyIsNull(qMax(murect.width(), murect.height())))
    #     return;
    #
    # const QRectF mbrect = painter->transform().mapRect(item->boundingRect());
    # if (qMin(mbrect.width(), mbrect.height()) < qreal(1.0))
    #     return;

    # ---------------------------------------------------------------------
    def update_painter_text_rect(self):
        r = QRectF(self._rect)
        size = self._rect.size()
        size.setWidth(self.sceneWidth() - self.textPadding)
        r.moveCenter(self._rect.center())
        self.painterTextRect = r

    # ---------------------------------------------------------------------
    def paint(self, painter, option, widget=None):

        # painter.setClipRect(self.boundingRect())
        # painter.setClipRect(option.exposedRect)
        # painter.setClipRegion(QRegion(option.exposedRect.toRect()))

        if not self._pen.width():
            painter.setPen(Qt.PenStyle.NoPen)
        else:
            painter.setPen(self._pen)
        # painter.setPen(QPen(self.cizgiRengi))
        painter.setBrush(self._brush)
        # !! eger kareyi burda cizersek, az sonra cizilen textin alphasinda problem oluyor.
        #   alphayi azalttikca yazi siyahlasiyor, en sonda 0 olunca kayboluyor.
        #   -- o yuzden texti cidikten sonra ciziyoruz bu kareyi deyip bunu iptal etmistik--
        #   ama self.cizgiRengini, QColor().fromrgb ile teker teker aynen yeniden olsuturunca
        #   ve bu yeni kopya renk ile pen olusturunca calısıyor. self.setCizgRengi nde bu islem,
        painter.drawRect(self._rect)

        if self._text:
            # painter.setWorldMatrixEnabled(False)
            painter.save()
            painter.setFont(self._font)
            # we recreate textPen from same exact color. otherwise, color's alpha not working.
            painter.setPen(self.textPen)
            painter.translate(self._rect.center())
            painter.rotate(-self.rotation())

            painter.translate(-self._rect.center())

            # ---------------------------------------------------------------------
            # --  basla --  elided text cizmek icin --
            # ---------------------------------------------------------------------
            # metrics = painter.fontMetrics()
            # txt = metrics.elidedText(self._text, Qt.ElideRight, self._rect.width() / scale)
            # # Qt.AlignCenter hem AlignVCenter hem de AlignHCenter icin yeterli
            # # ayrica en fazla iki tane kullanilabiliyormus, ve AlignCenter 2 tane sayiliyormus.
            # # painter.drawText(self._rect, Qt.AlignCenter | Qt.AlignVCenter, txt)
            # painter.drawText(self._rect, Qt.AlignCenter, txt)
            # ---------------------------------------------------------------------
            # ---  bitir --- elided text cizmek icin --
            # ---------------------------------------------------------------------

            # painter.drawRect(r)
            # painter.drawText(self._rect, self._text, self.painterTextOption)
            painter.drawText(self.painterTextRect, self._text, self.painterTextOption)

            # painter.drawText(10,10, txt)
            painter.restore()
            # painter.setWorldMatrixEnabled(True)

        # if option.state & QStyle.State_MouseOver:
        if option.state & QStyle.StateFlag.State_Selected or self.cosmeticSelect:
            # painter.setPen(QPen(option.palette.windowText(), 0, Qt.DashLine))
            # painter.setPen(QPen(option.palette.highlight(), 0, Qt.DashLine))

            if self.isActiveItem:
                selectionPenBottom = self.selectionPenBottomIfAlsoActiveItem
                if not self.isPinned:
                    painter.setPen(Qt.PenStyle.NoPen)
                    painter.setBrush(self.handleBrush)
                    painter.drawRect(self.topLeftHandle)
                    painter.drawRect(self.topRightHandle)
                    painter.drawRect(self.bottomRightHandle)
                    painter.drawRect(self.bottomLeftHandle)
            else:
                selectionPenBottom = self.selectionPenBottom

            painter.setBrush(Qt.BrushStyle.NoBrush)

            painter.setPen(selectionPenBottom)
            painter.drawRect(self._rect)

            # painter.setPen(self.selectionPenTop)
            # painter.drawRect(self._rect)

        # if option.state & QStyle.State_MouseOver:
        #     #######################################################################
        #     # !!! simdilik iptal, gorsel fazlalik olusturmakta !!!
        #     #######################################################################
        #     # if not self.isPinned and self.isActiveItem:
        #     # painter.setPen(self.handlePen)
        #     painter.setPen(Qt.PenStyle.NoPen)
        #     painter.setBrush(self.handleBrush)
        #     painter.drawRect(self.topLeftHandle)
        #     painter.drawRect(self.topRightHandle)
        #     painter.drawRect(self.bottomRightHandle)
        #     painter.drawRect(self.bottomLeftHandle)

            # painter.setPen(QPen(Qt.GlobalColor.red, 3))
            # painter.drawPoint(self.topLeftHandle.center())
            # painter.drawPoint(self.topRightHandle.center())
            # painter.drawPoint(self.bottomRightHandle.center())
            # painter.drawPoint(self.bottomLeftHandle.center())

        # if option.state & QStyle.StateFlag.State_MouseOver:
        #     painter.setBrush(Qt.BrushStyle.NoBrush)
        #     painter.setPen(self.selectionPenBottom)
        #     painter.drawRect(self._rect)

            #######################################################################

        # font = painter.font()
        # font.setPointSizeF(self.fontPointSizeF)
        # painter.setFont(font)

        # # karenin altina yazsin yaziyi amcli ama bounding bozu degistirmek lazim.
        # # klasin simdilik.
        # # metrics.size() # this is metrics.boundingRect() size.
        # rect = metrics.boundingRect(self.text)
        # rect.moveTop(brect.bottom())
        # painter.drawText(rect, Qt.AlignCenter, self.text)

        # # # # # # debug start - pos() # # # # #
        # p = self.pos()
        # s = self.scenePos()
        # painter.drawText(self._rect,
        #                  "{0:.2f},  {1:.2f} pos \n{2:.2f},  {3:.2f} spos".format(p.x(), p.y(), s.x(), s.y()))
        # # # # t = self.transformOriginPoint()
        # # # painter.drawRect(t.x()-12, t.y()-12,24,24)
        # mapped = self.mapToScene(self._rect.topLeft())
        # painter.drawText(self._rect.x(), self._rect.y(), "{0:.2f}  {1:.2f} map".format(mapped.x(), mapped.y()))
        # painter.drawEllipse(self.scenePos(), 10, 10)
        # painter.setPen(Qt.blue)
        # painter.drawEllipse(self.mapFromScene(self.pos()), 10, 10)
        # r = self.textItem.boundingRect()
        # r = self.mapRectFromItem(self.textItem, r)
        # painter.drawRect(r)
        # painter.drawText(self._rect.center(), "{0:f}  {1:f}".format(self.sceneWidth(), self.sceneHeight()))
        # painter.setPen(QPen(Qt.red,17))
        # painter.drawPoint(self._rect.center())
        # painter.setPen(QPen(Qt.green,12))
        # painter.drawPoint(self.mapFromScene(self.sceneBoundingRect().center()))
        # painter.setPen(QPen(Qt.blue,8))
        # painter.drawPoint(self.sceneBoundingRect().center())
        # painter.drawRect(self.painterTextRect)
        # painter.drawRect(self.childrenBoundingRect())
        # painter.drawRect(self.boundingRect())
        # painter.setPen(Qt.green)
        # painter.drawRect(self.boundingRect() | self.childrenBoundingRect())
        # painter.setPen(QPen(Qt.green, 8))
        # painter.drawPoint(self.mapFromScene(self.sceneBoundingRect().center()))
        # painter.drawPoint(0, 0)
        # painter.setPen(QPen(Qt.red, 16))
        # painter.drawPoint(self.mapFromScene(self.pos()))
        # painter.setPen(QPen(Qt.blue, 12))
        # painter.drawPoint(self.mapFromScene(self.scenePos()))
        # # # # # # debug end - pos() # # # # #

    # ---------------------------------------------------------------------
    def itemChange(self, change, value):
        # if change == self.ItemPositionChange:
        # if change == self.ItemSelectedChange:
        # if change == self.ItemSelectedHasChanged:
        if change == QGraphicsItem.GraphicsItemChange.ItemSelectedChange:
            if value:
                self.scene().parent().item_selected(self)
            else:
                self.scene().parent().item_deselected(self)

        # if change == self.ItemSceneChange:
        #     print(value, "eveeeet")
        # elif change == self.ItemParentChange:
        #     if value:
        #         if value.type() == QGraphicsItem.UserType + 7:  # this is Group's custom type value,
        #             print("evet")

        # return QGraphicsRectItem.itemChange(self, change, value)
        return value
        # super(Rect, self).itemChange(change, value)

    # ---------------------------------------------------------------------
    def contextMenuEvent(self, event):
        # TODO: bu alttaki iki satir aktif idi, ve de ctrl harici sag tiklayinca
        # (base- text -path -group nesnelerinin contextMenuEventinde var)
        # mesela birden fazla nesne secili ve de gruplayacagız diyelim sag menu ile
        # ctrl basılı degil ise tikladigimiz secili kaliyor digerleri siliniyordu
        # uygunsuz bir kullanıcı deneyimi, niye yaptık acaba boyleyi hatırlayana kadar kalsın burda :)
        # if not event.modifiers() == Qt.KeyboardModifier.ControlModifier:
        #     self.scene().clearSelection()
        self.setSelected(True)

        # self.scene().parent().item_context_menu(self, event.screenPos())
        self.scene().parent().on_nesne_sag_menu_about_to_show(self)
        self.scene().parent().nesneSagMenu.popup(event.screenPos())
        event.accept()

    # ---------------------------------------------------------------------
    def mouseDoubleClickEvent(self, event):
        if not self.tempTextItem:
            self.tempTextItem = TempTextItem(self.sceneWidth() - self.textPadding,
                                             self._font,
                                             self.yaziRengi,
                                             parent=self)
            self.tempTextItem.setParentItem(self)

        self.tempTextItem.setPlainText(self.text())
        self.tempEskiText = self.text()
        self.setText("")
        self.tempTextItem.setRotation(360 - self.rotation())
        # self.tempTextItem.rotateWithOffset(360 - self.rotation())
        self.tempTextItem.setCenter(self._rect.center())
        self.tempTextItem.setFocus()

        # !! burda lambda kullanmazsak, self.is_temp_text_item_empty ,self daha olsumadigi icin,
        # selfi null olarak goruyor ve baglanti kuramiyor.
        self.tempTextItem.textItemFocusedOut.connect(lambda: self.temp_text_item_focused_out())

        super(BaseItem, self).mouseDoubleClickEvent(event)

    # ---------------------------------------------------------------------
    @Slot()
    def temp_text_item_focused_out(self):
        tmpText = self.tempTextItem.toPlainText()
        self.scene().undoRedo.undoableItemSetText(self.scene().undoStack, "change text", self,
                                                  self.tempEskiText, tmpText)
        # if not tmpText:
        self.scene().removeItem(self.tempTextItem)
        self.tempTextItem.deleteLater()
        self.tempTextItem = None

    # ---------------------------------------------------------------------
    # def keyPressEvent(self, event):
    #
    #     super(BaseItem, self).keyPressEvent(event)

    # ---------------------------------------------------------------------
    def mousePressEvent(self, event):

        self._resizing = False  # we could use "self._resizingFrom = 0" instead, but self._resizing is more explicit.
        self._fixedResizePoint = None
        self._resizingFrom = None
        self._eskiRectBeforeResize = None

        self._eskiPosBeforeResize = self.pos()
        # self._eskiPosBeforeResize = self.scenePos()

        if self.topLeftHandle.contains(event.pos()):
            self._resizing = True
            self._fixedResizePoint = self._rect.bottomRight()
            self._resizingFrom = 1
        elif self.topRightHandle.contains(event.pos()):
            self._resizing = True
            self._fixedResizePoint = self._rect.bottomLeft()
            self._resizingFrom = 2
        elif self.bottomRightHandle.contains(event.pos()):
            self._resizing = True
            self._fixedResizePoint = self._rect.topLeft()
            self._resizingFrom = 3
        elif self.bottomLeftHandle.contains(event.pos()):
            self._resizing = True
            self._fixedResizePoint = self._rect.topRight()
            self._resizingFrom = 4
        self._eskiRectBeforeResize = self._rect

        super(BaseItem, self).mousePressEvent(event)

        # TODO: bu kozmetik amacli, sadece release de olunca gecikmeli seciyormus hissi oluyor
        if self.isSelected() and self.childItems():
            self.scene().select_all_children_recursively(self, cosmeticSelect=True)

        # self.scene().deactivate_item(self)
        # self.scene().activate_item(self)

    # ---------------------------------------------------------------------
    def mouseReleaseEvent(self, event):
        if self._resizing:
            # sahneye ilk cizim kontrolu,(undoya eklemesin diye)
            if not self._eskiRectBeforeResize.size() == QSizeF(1, 1):
                rect = self._rect
                # self.setPos(self.mapToScene(rect.topLeft()))
                # rect.moveTo(0, 0)
                self.scene().undoRedo.undoableResizeBaseItem(self.scene().undoStack,
                                                             "resize",
                                                             self,
                                                             # yeniRect=self._rect,
                                                             yeniRect=rect,
                                                             eskiRect=self._eskiRectBeforeResize,
                                                             eskiPos=self._eskiPosBeforeResize)

            self._resizing = False

            self.update_painter_text_rect()
            self.scene().unite_with_scene_rect(self.sceneBoundingRect())
        super(BaseItem, self).mouseReleaseEvent(event)

        if self.childItems():
            if self.isSelected():
                self.scene().select_all_children_recursively(self, cosmeticSelect=False, topmostParent=True)
            else:
                self.scene().select_all_children_recursively(self, cosmeticSelect=False)

    # ---------------------------------------------------------------------
    def mouseMoveEvent(self, event):

        # TODO: burda QRectF().normalized da kullanilabilir belki..

        if self._resizing:
            # cunku px=1 , py=1 olarak basliyor cizmeye. burda sifirliyoruz eger ilk sahneye ekleme cizimi ise.
            if not self._eskiRectBeforeResize.size() == QSizeF(1, 1):
                px = self._fixedResizePoint.x()
                py = self._fixedResizePoint.y()
            else:
                px = py = 0
            # mx = event.scenePos().x()
            # my = event.scenePos().y()
            mx = event.pos().x()
            my = event.pos().y()
            topLeft = QPointF(min(px, mx), min(py, my))  # top-left corner (x,y)
            bottomRight = QPointF(max(px, mx), max(py, my))  # bottom-right corner (x,y)
            # size = QSizeF(fabs(mx - px), fabs(my - py))
            # rect = QRectF(topLeft, size)

            rect = QRectF(topLeft, bottomRight)

            c = self._rect.center()

            # Alt Key - to resize around center.
            # if event.modifiers() & Qt.AltModifier:
            if event.modifiers() == Qt.KeyboardModifier.AltModifier:
                rect.moveCenter(c)

            # ---------------------------------------------------------------------
            #  Ctrl Key - to keep aspect ratio while resizing.
            if event.modifiers() == Qt.KeyboardModifier.ControlModifier:
                tl = self._rect.topLeft()
                tr = self._rect.topRight()
                br = self._rect.bottomRight()
                bl = self._rect.bottomLeft()
                c = self._rect.center()

                yeniSize = rect.size()
                eskiSize = self._rect.size()
                # eskiSize.scale(yeniSize, Qt.KeepAspectRatio)
                eskiSize.scale(yeniSize.height(), yeniSize.height(), Qt.AspectRatioMode.KeepAspectRatioByExpanding)
                # eskiSize.scale(yeniSize.height(), yeniSize.height(), Qt.KeepAspectRatio)

                # if not eskiSize.isNull():
                if not eskiSize.isEmpty():
                    self.yedekSize = QSizeF(eskiSize)

                else:
                    eskiSize = QSizeF(self.yedekSize)

                rect.setSize(eskiSize)
                h = rect.height()
                w = rect.width()

                if self._resizingFrom == 1:
                    rect.moveTopLeft(QPointF(br.x() - w, br.y() - h))

                if self._resizingFrom == 2:
                    rect.moveTopRight(QPointF(bl.x() + w, bl.y() - h))

                if self._resizingFrom == 3:
                    rect.moveBottomRight(QPointF(tl.x() + w, tl.y() + h))

                elif self._resizingFrom == 4:
                    rect.moveBottomLeft(QPointF(tr.x() - w, tr.y() + h))

                # Alt Key - to resize around center
                if event.modifiers() == Qt.KeyboardModifier.AltModifier:
                    rect.moveCenter(c)

            # ---------------------------------------------------------------------
            # Shift Key - to make square (equals width and height)
            if event.modifiers() == Qt.KeyboardModifier.ShiftModifier:
                height = my - py
                if self._resizingFrom == 1:
                    rect.setCoords(px + height, py + height, px, py)
                    rect = rect.normalized()

                elif self._resizingFrom == 2:
                    rect.setCoords(px, py + height, px - height, py)
                    rect = rect.normalized()

                elif self._resizingFrom == 3:
                    rect.setCoords(px, py, px + height, py + height)
                    rect = rect.normalized()

                elif self._resizingFrom == 4:
                    rect.setCoords(px - height, py, px, py + height)
                    rect = rect.normalized()

                # Alt Key - to resize around center
                if event.modifiers() == Qt.KeyboardModifier.AltModifier:
                    rect.moveCenter(c)

            self.setRect(rect)  # mouse release eventten gonderiyoruz undoya
            self.update_resize_handles()
            self.scene().parent().change_transform_box_values(self)
            # self.setPos(x, y)
            # return QGraphicsItem.mouseMoveEvent(self, event)

            self.update_painter_text_rect()

        # event.accept()
        else:
            super(BaseItem, self).mouseMoveEvent(event)

        # super(BaseItem, self).mouseMoveEvent(event)

    # ---------------------------------------------------------------------
    def hoverEnterEvent(self, event):
        # event propagationdan dolayi bos da olsa burda implement etmek lazim
        # mousePress,release,move,doubleclick de bu mantıkta...
        # baska yerden baslayip mousepress ile , mousemove baska, mouseReleas baska widgetta gibi
        # icinden cikilmaz durumlara sebep olabiliyor.
        # ayrica override edince accept() de edilmis oluyor mouseeventler
        super(BaseItem, self).hoverEnterEvent(event)

    # ---------------------------------------------------------------------
    def hoverMoveEvent(self, event):

        # cursor = self.scene().parent().cursor()

        # if self.isSelected() and self.scene().aktifArac == self.scene().SecimAraci:
        if not self.ustGrup:
            if self.scene().aktifArac == self.scene().SecimAraci:
                if self.topLeftHandle.contains(event.pos()) or self.bottomRightHandle.contains(event.pos()):
                    self.scene().parent().setCursor(Qt.CursorShape.SizeFDiagCursor, gecici_mi=True)
                    # self.setCursor(Qt.SizeFDiagCursor, gecici_mi=True)
                elif self.topRightHandle.contains(event.pos()) or self.bottomLeftHandle.contains(event.pos()):
                    self.scene().parent().setCursor(Qt.CursorShape.SizeBDiagCursor, gecici_mi=True)
                    # self.setCursor(Qt.SizeBDiagCursor, gecici_mi=True)
                else:
                    self.scene().parent().setCursor(self.scene().parent().imlec_arac, gecici_mi=True)

        super(BaseItem, self).hoverMoveEvent(event)

    # ---------------------------------------------------------------------
    def hoverLeaveEvent(self, event):
        # if self.isSelected():
        # self.setCursor(self.saved_cursor)
        if not self.ustGrup:
            self.scene().parent().setCursor(self.scene().parent().imlec_arac, gecici_mi=True)

        super(BaseItem, self).hoverLeaveEvent(event)

    # ---------------------------------------------------------------------
    def wheelEvent(self, event):
        # factor = 1.41 ** (event.delta() / 240.0)

        if self.ustGrup:
            return QGraphicsItem.wheelEvent(self, event)

        toplam = event.modifiers().value

        # ctrl = int(Qt.ControlModifier)
        shift = Qt.KeyboardModifier.ShiftModifier.value
        alt = Qt.KeyboardModifier.AltModifier.value

        ctrlAlt = Qt.KeyboardModifier.ControlModifier.value + Qt.KeyboardModifier.AltModifier.value
        ctrlShift = Qt.KeyboardModifier.ControlModifier.value + Qt.KeyboardModifier.ShiftModifier.value
        altShift = Qt.KeyboardModifier.AltModifier.value + Qt.KeyboardModifier.ShiftModifier.value
        ctrlAltShift = Qt.KeyboardModifier.ControlModifier.value + Qt.KeyboardModifier.AltModifier.value + Qt.KeyboardModifier.ShiftModifier.value

        # if event.modifiers() & Qt.ControlModifier:

        # elif event.modifiers() & Qt.ShiftModifier:
        if toplam == shift:
            self.changeFontSizeF(event.delta())

        # elif event.modifiers() & Qt.AltModifier:
        elif toplam == alt:
            self.changeBackgroundColorAlpha(event.delta())

        elif toplam == ctrlShift:
            # self.scaleItem(event.delta())
            self.scaleItemByResizing(event.delta())

        elif toplam == ctrlAlt:
            self.changeTextColorAlpha(event.delta())

        elif toplam == altShift:
            # self.changeImageItemTextBackgroundColorAlpha(event.delta())
            self.changeLineColorAlpha(event.delta())

        elif toplam == ctrlAltShift:
            self.rotateItem(event.delta())

        # elif toplam == ctrlAltShift:
        #     self.scaleItemByResizing(event.delta())

        else:
            super(BaseItem, self).wheelEvent(event)

    # ---------------------------------------------------------------------
    def changeFontSizeF(self, delta):

        if self.tempTextItem:
            return

        # font = self.font()
        size = self.fontPointSizeF()

        if delta > 0:
            # font.setPointSizeF(size + 1)
            size += 1

        else:
            if size > 3:
                # font.setPointSizeF(size - 1)
                size -= 1
            else:
                # undolari biriktermesin diye donuyoruz,
                # yoksa zaten ayni size de yeni bir undolu size komutu veriyor.
                return

        # self.setFont(font)

        if self.childItems():
            self.scene().undoStack.beginMacro("change text size")
            self.scene().undoRedo.undoableSetFontSizeF(self.scene().undoStack, "change text size", self, size)
            for c in self.childItems():
                c.changeFontSizeF(delta)
            self.scene().undoStack.endMacro()
        else:
            self.scene().undoRedo.undoableSetFontSizeF(self.scene().undoStack, "change text size", self, size)

    # ---------------------------------------------------------------------
    def scaleItemByResizing(self, delta):

        if self.tempTextItem:
            return

        scaleFactor = 1.1
        if delta < 0:
            scaleFactor = 1 / scaleFactor

        # eskiTamCerceve = self.boundingRect() | self.childrenBoundingRect()
        center = self._rect.center()

        rect = QRectF(self._rect.topLeft(), self._rect.size() * scaleFactor)
        rect.moveCenter(center)
        yeniPos = self.mapToScene(rect.topLeft())
        if self.parentItem():
            yeniPos = self.mapToParent(rect.topLeft())
        rect.moveTo(0, 0)

        self.scene().undoRedo.undoableScaleBaseItemByResizing(self.scene().undoStack,
                                                              "_scale by resizing",
                                                              self, rect, scaleFactor,
                                                              yeniPos)

    # ---------------------------------------------------------------------
    def repositionChildItems(self, diff):
        # TODO: item rotation 0 olmadigi durumlarda calismiyor , kaymalar oluyor.
        for c in self.childItems():
            c.moveBy(-diff.x(), -diff.y())

    # ---------------------------------------------------------------------
    def rotateWithOffset(self, angle):
        # Thanks to...
        # https://stackoverflow.com/a/32734103
        # ayrica bu rotate with offsetlerde mod 360 yapmiyoruz
        # cunku buraya gelen illaki modlanmis geliyor.
        # bunu niye yazdim. undoya gitmeden rotate olabiliyor bununla,
        # (undoRotatelerde mod var.)
        cEski = self.sceneCenter()
        self.setRotation(angle)
        cYeni = self.sceneCenter()
        diff = cEski - cYeni
        self.moveBy(diff.x(), diff.y())
        self.update_painter_text_rect()

    # ---------------------------------------------------------------------
    def rotateItem(self, delta):

        if delta > 0:
            # self.setRotation(self.rotation() + 5)
            self.scene().undoRedo.undoableRotateWithOffset(self.scene().undoStack, "rotate", self,
                                                           self.rotation() + 5)
            # self.rotateWithOffset(self.rotation() + 5, self)

        else:
            # self.setRotation(self.rotation() - 5)
            self.scene().undoRedo.undoableRotateWithOffset(self.scene().undoStack, "rotate", self,
                                                           self.rotation() - 5)
            # self.rotateWithOffset(self.rotation() - 5, self)
        # self.update_painter_text_rect()
        # self.update()

    # ---------------------------------------------------------------------
    def changeBackgroundColorAlpha(self, delta):

        if self.tempTextItem:
            return

        col = shared.calculate_alpha(delta, QColor(self.arkaPlanRengi))

        if self.childItems():
            self.scene().undoStack.beginMacro("change items' background alpha")
            self.scene().undoRedo.undoableSetItemBackgroundColorAlpha(self.scene().undoStack,
                                                                      "_change item's background alpha",
                                                                      self, col)
            for c in self.childItems():
                c.changeBackgroundColorAlpha(delta)
            self.scene().undoStack.endMacro()
        else:
            self.scene().undoRedo.undoableSetItemBackgroundColorAlpha(self.scene().undoStack,
                                                                      "change item's background alpha",
                                                                      self, col)

    # ---------------------------------------------------------------------
    def changeLineColorAlpha(self, delta):

        if self.tempTextItem:
            return

        # col = shared.calculate_alpha(delta, self._pen.color())
        col = shared.calculate_alpha(delta, self.cizgiRengi)

        if self.childItems():
            self.scene().undoStack.beginMacro("change items' line alpha")
            self.scene().undoRedo.undoableSetLineColorAlpha(self.scene().undoStack,
                                                            "_change item's line alpha", self, col)
            for c in self.childItems():
                c.changeLineColorAlpha(delta)
            self.scene().undoStack.endMacro()
        else:
            self.scene().undoRedo.undoableSetLineColorAlpha(self.scene().undoStack,
                                                            "change item's line alpha", self, col)

    # ---------------------------------------------------------------------
    def changeTextColorAlpha(self, delta):

        if self.tempTextItem:
            return

        col = shared.calculate_alpha(delta, self.yaziRengi)

        if self.childItems():
            self.scene().undoStack.beginMacro("change items' text alpha")
            self.scene().undoRedo.undoableSetTextColorAlpha(self.scene().undoStack,
                                                            "_change item's text alpha", self, col)
            for c in self.childItems():
                c.changeTextColorAlpha(delta)
            self.scene().undoStack.endMacro()
        else:
            self.scene().undoRedo.undoableSetTextColorAlpha(self.scene().undoStack,
                                                            "change item's text alpha", self, col)

    # ---------------------------------------------------------------------
    def changeImageItemTextBackgroundColorAlpha(self, delta):
        # grup nesnesinden cagriliyor alt+shift
        if self.childItems():
            startedMacro = False
            for c in self.childItems():
                if c.type() == shared.IMAGE_ITEM_TYPE:
                    if not startedMacro:
                        self.scene().undoStack.beginMacro("change image items' text background alpha")
                        startedMacro = True
                    c.changeImageItemTextBackgroundColorAlpha(delta)
            if startedMacro:
                self.scene().undoStack.endMacro()
